@@ -26,8 +26,7 @@ AccountDialog::AccountDialog(PumpController *pumpController, QWidget *parent) :
     this->setWindowIcon(QIcon::fromTheme("dialog-password"));
     this->setWindowFlags(Qt::Window);
     this->setWindowModality(Qt::ApplicationModal);
-    this->setMinimumSize(540, 180);
-    this->resize(800, 180);
+    this->setMinimumSize(640, 320);
 
     this->pController = pumpController;
 
@@ -36,6 +35,16 @@ AccountDialog::AccountDialog(PumpController *pumpController, QWidget *parent) :
     connect(pController, SIGNAL(authorizationStatusChanged(bool)),
             this, SLOT(showAuthorizationStatus(bool)));
 
+    QFont helpFont;
+    helpFont.setPointSize(helpFont.pointSize() - 1);
+
+    this->helpMessage1Label = new QLabel(tr("First, enter your Webfinger ID, your pump.io address.") + "\n"
+                                         + tr("Your address looks like username@pumpserver.org, and "
+                                              "you can find it in your profile, in the web interface.") + "\n"
+                                         + tr("If your profile is at https://pump.example/yourname, then "
+                                              "your address is yourname@pump.example"));
+    helpMessage1Label->setWordWrap(true);
+    helpMessage1Label->setFont(helpFont);
 
     userIDIconLabel = new QLabel();
     userIDIconLabel->setPixmap(QIcon::fromTheme("preferences-desktop-user").pixmap(32,32));
@@ -46,9 +55,21 @@ AccountDialog::AccountDialog(PumpController *pumpController, QWidget *parent) :
 
     getVerifierButton = new QPushButton(QIcon::fromTheme("object-unlocked"),
                                         tr("Get &Verifier Code"));
+    getVerifierButton->setToolTip(tr("After clicking this button, a web browser will "
+                                     "open, requesting authorization for Dianara"));
     connect(getVerifierButton, SIGNAL(clicked()),
             this, SLOT(askForToken()));
 
+
+    this->helpMessage2Label = new QLabel(tr("Once you have authorized Dianara from your Pump server "
+                                            "web interface, you'll receive a code called VERIFIER.\n"
+                                            "Copy it and paste it into the field below.",
+                                            // Comment for the translators
+                                            "Don't translate the VERIFIER word!"));
+    helpMessage2Label->setWordWrap(true);
+    helpMessage2Label->setFont(helpFont);
+
+
     verifierIconLabel = new QLabel();
     verifierIconLabel->setPixmap(QIcon::fromTheme("dialog-password").pixmap(32,32));
     verifierLabel = new QLabel(tr("Verifier code:"));
@@ -73,8 +94,14 @@ AccountDialog::AccountDialog(PumpController *pumpController, QWidget *parent) :
             errorsLabel, SLOT(clear()));
 
 
+    QFont authorizationStatusFont;
+    authorizationStatusFont.setPointSize(authorizationStatusFont.pointSize() + 2);
+    authorizationStatusFont.setBold(true);
+
     authorizationStatusLabel = new QLabel();
-    authorizationStatusLabel->setTextInteractionFlags(Qt::TextSelectableByKeyboard | Qt::TextSelectableByMouse);
+    authorizationStatusLabel->setTextInteractionFlags(Qt::TextSelectableByKeyboard
+                                                      | Qt::TextSelectableByMouse);
+    authorizationStatusLabel->setFont(authorizationStatusFont);
 
 
     saveButton = new QPushButton(QIcon::fromTheme("document-save"),
@@ -86,23 +113,44 @@ AccountDialog::AccountDialog(PumpController *pumpController, QWidget *parent) :
     connect(cancelButton, SIGNAL(clicked()),
             this, SLOT(hide()));
 
-    mainLayout = new QGridLayout();
-    mainLayout->setAlignment(Qt::AlignLeft);
-    mainLayout->addWidget(userIDIconLabel,   0, 0);
-    mainLayout->addWidget(userIDLabel,       0, 1);
-    mainLayout->addWidget(userIDLineEdit,    0, 2);
-    mainLayout->addWidget(getVerifierButton, 0, 3);
 
-    mainLayout->addWidget(verifierIconLabel,          1, 0);
-    mainLayout->addWidget(verifierLabel,              1, 1);
-    mainLayout->addWidget(verifierLineEdit,           1, 2);
-    mainLayout->addWidget(authorizeApplicationButton, 1, 3);
+    // Layout
+    this->mainLayout = new QVBoxLayout();
+    mainLayout->addWidget(helpMessage1Label, 3);
+    mainLayout->addStretch(2);
 
-    mainLayout->addWidget(authorizationStatusLabel,   2, 0, 1, 4, Qt::AlignCenter);
-    mainLayout->addWidget(errorsLabel,       3, 0, 1, 4, Qt::AlignCenter);
+    this->idLayout = new QHBoxLayout();
+    idLayout->addWidget(userIDIconLabel);
+    idLayout->addSpacing(4);
+    idLayout->addWidget(userIDLabel);
+    idLayout->addWidget(userIDLineEdit);
+    idLayout->addWidget(getVerifierButton);
+    mainLayout->addLayout(idLayout, 3);
+    mainLayout->addStretch(3);
+    mainLayout->addSpacing(8);
+
+    mainLayout->addWidget(helpMessage2Label, 3);
+    mainLayout->addStretch(2);
+
+    this->verifierLayout = new QHBoxLayout();
+    verifierLayout->addWidget(verifierIconLabel);
+    verifierLayout->addSpacing(4);
+    verifierLayout->addWidget(verifierLabel);
+    verifierLayout->addWidget(verifierLineEdit);
+    verifierLayout->addWidget(authorizeApplicationButton);
+    mainLayout->addLayout(verifierLayout, 3);
+
+    mainLayout->addWidget(errorsLabel, 1, Qt::AlignCenter);
+    mainLayout->addWidget(authorizationStatusLabel, 1, Qt::AlignCenter);
+    mainLayout->addStretch(2);
+    mainLayout->addSpacing(8);
+
+    this->buttonsLayout = new QHBoxLayout();
+    buttonsLayout->setAlignment(Qt::AlignRight);
+    buttonsLayout->addWidget(saveButton);
+    buttonsLayout->addWidget(cancelButton);
+    mainLayout->addLayout(buttonsLayout, 1);
 
-    mainLayout->addWidget(saveButton,        4, 2, 1, 1, Qt::AlignRight);
-    mainLayout->addWidget(cancelButton,      4, 3, 1, 1, Qt::AlignLeft);
     this->setLayout(mainLayout);
 
 
@@ -188,7 +236,8 @@ void AccountDialog::showAuthorizationStatus(bool authorized)
 {
     if (authorized)
     {
-        this->authorizationStatusLabel->setText("[[ " + tr("Dianara is authorized to access your data") + " ]]");
+        this->authorizationStatusLabel->setText(QString::fromUtf8("\342\234\224 ") // Check mark
+                                                + tr("Dianara is authorized to access your data"));
     }
     else
     {
