@@ -61,7 +61,7 @@ AudienceSelector::AudienceSelector(PumpController *pumpController,
     allContactsListWidget->setDefaultDropAction(Qt::CopyAction);
     allContactsListWidget->setSelectionMode(QListView::ExtendedSelection);
     allContactsListWidget->setSortingEnabled(true);
-    connect(allContactsListWidget, SIGNAL(doubleClicked(QModelIndex)),
+    connect(allContactsListWidget, SIGNAL(activated(QModelIndex)),
             this, SLOT(copyClickedToSelected(QModelIndex)));
 
     addToSelectedButton = new QPushButton(QIcon::fromTheme("list-add"),
@@ -84,7 +84,7 @@ AudienceSelector::AudienceSelector(PumpController *pumpController,
     // Right side, selected contacts
 
     explanationLabel = new QLabel(tr("Select people from the list on the left.\n"
-                                     "You can drag them with the mouse, "
+                                     "You can drag them with the mouse, click or "
                                      "double-click on them, or select them and "
                                      "use the button below."));
     explanationLabel->setWordWrap(true);
@@ -211,9 +211,9 @@ void AudienceSelector::updateAllContactsList(QString listType,
     QString singleContactString;
     foreach (QVariant contact, contactsVariantList)
     {
-        singleContactString = contact.toMap()["displayName"].toString();
+        singleContactString = contact.toMap().value("displayName").toString();
         singleContactString.append("  <");
-        singleContactString.append(contact.toMap()["id"].toString().remove(0,5));
+        singleContactString.append(contact.toMap().value("id").toString().remove(0,5));
         singleContactString.append(">");
 
         allContactsStringList.append(singleContactString);
