@@ -24,17 +24,24 @@ Comment::Comment(PumpController *pumpController,
                  QVariantMap commentMap,
                  QWidget *parent) : QFrame(parent)
 {
-    this->setSizePolicy(QSizePolicy::Minimum,
-                        QSizePolicy::Minimum);
-
     this->pController = pumpController;
-    this->commentID = commentMap["id"].toString();
-    this->objectType = commentMap["objectType"].toString();
-    QVariantMap commentAuthorMap = commentMap["author"].toMap();
-    commentIsOwn = false;
 
 
-    QString commentAuthorID = commentAuthorMap["id"].toString();
+    QSizePolicy sizePolicy;
+    sizePolicy.setHeightForWidth(false);
+    sizePolicy.setWidthForHeight(false);
+    //sizePolicy.setHorizontalStretch(1);
+    //sizePolicy.setVerticalStretch(6);
+    sizePolicy.setHorizontalPolicy(QSizePolicy::MinimumExpanding);
+    sizePolicy.setVerticalPolicy(QSizePolicy::MinimumExpanding);
+    this->setSizePolicy(sizePolicy);
+
+
+    this->commentID = commentMap.value("id").toString();
+    this->objectType = commentMap.value("objectType").toString();
+    QVariantMap commentAuthorMap = commentMap.value("author").toMap();
+
+    QString commentAuthorID = commentAuthorMap.value("id").toString();
     commentAuthorID.remove(0, 5); // remove "acct:" at the beginning
     if (commentAuthorID == pController->currentUserID())
     {
@@ -45,6 +52,8 @@ Comment::Comment(PumpController *pumpController,
     }
     else
     {
+        commentIsOwn = false;
+
         this->setFrameStyle(QFrame::Raised | QFrame::StyledPanel);
     }
 
@@ -53,8 +62,8 @@ Comment::Comment(PumpController *pumpController,
     // Avatar pixmap
     avatarLabel = new QLabel();
     avatarLabel->setToolTip(commentAuthorID);
-    QString commentAuthorAvatarURL = commentAuthorMap["image"].toMap()["url"].toString();
-    QString commentAuthorAvatarFile = MiscHelpers::getCachedAvatarFilename(commentAuthorAvatarURL);
+    this->commentAuthorAvatarUrl = commentAuthorMap.value("image").toMap().value("url").toString();
+    QString commentAuthorAvatarFile = MiscHelpers::getCachedAvatarFilename(commentAuthorAvatarUrl);
     if (QFile::exists(commentAuthorAvatarFile))
     {
         avatarLabel->setPixmap(QPixmap(commentAuthorAvatarFile)
@@ -65,7 +74,9 @@ Comment::Comment(PumpController *pumpController,
         // Set placeholder
         avatarLabel->setPixmap(QIcon::fromTheme("user-identity").pixmap(32, 32));
         // and download avatar for next time
-        this->pController->getAvatar(commentAuthorAvatarURL);
+        connect(pController, SIGNAL(avatarStored(QString,QString)),
+                this, SLOT(redrawAvatar(QString,QString)));
+        this->pController->getAvatar(commentAuthorAvatarUrl);
     }
 
 
@@ -76,7 +87,7 @@ Comment::Comment(PumpController *pumpController,
     // Name, with ID as tooltip
     commentsFont.setBold(true);
 
-    fullNameLabel = new QLabel(commentAuthorMap["displayName"].toString());
+    fullNameLabel = new QLabel(commentAuthorMap.value("displayName").toString());
     fullNameLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
     fullNameLabel->setFont(commentsFont);
     fullNameLabel->setToolTip(commentAuthorID);
@@ -86,7 +97,7 @@ Comment::Comment(PumpController *pumpController,
     commentsFont.setBold(false);
     commentsFont.setItalic(true);
 
-    QString timestamp = commentMap["published"].toString();
+    QString timestamp = commentMap.value("published").toString();
     QString commentExactTime = Timestamp::localTimeDate(timestamp);
     QString commentFuzzyTime = Timestamp::fuzzyTime(timestamp);
     timestampLabel = new QLabel(commentFuzzyTime);
@@ -108,9 +119,19 @@ Comment::Comment(PumpController *pumpController,
     connect(likeLabel, SIGNAL(linkActivated(QString)),
             this, SLOT(likeComment(QString)));
 
-    commentIsLiked = commentMap["liked"].toBool();
+    commentIsLiked = commentMap.value("liked").toBool();
     this->fixLikeLabelText();
 
+
+    quoteLabel = new QLabel("<a href=\"quote://\">" + tr("Quote", "This is a verb, infinitive") + "</a>");
+    quoteLabel->setContextMenuPolicy(Qt::NoContextMenu);
+    quoteLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
+    quoteLabel->setFont(commentsFont);
+    quoteLabel->setToolTip(tr("Reply quoting this comment"));
+    connect(quoteLabel, SIGNAL(linkActivated(QString)),
+            this, SLOT(quoteComment()));
+
+
     deleteLabel = new QLabel("<a href=\"delete://\">" + tr("Delete") + "</a>");
     deleteLabel->setContextMenuPolicy(Qt::NoContextMenu);
     deleteLabel->setAlignment(Qt::AlignTop | Qt::AlignRight);
@@ -126,26 +147,30 @@ Comment::Comment(PumpController *pumpController,
     commentsFont.setBold(false);
     likesCountLabel->setFont(commentsFont);
 
-    this->setLikesCount(commentMap["likes"].toMap()["totalItems"].toInt(),
-                        commentMap["likes"].toMap()["items"].toList());
+    this->setLikesCount(commentMap.value("likes").toMap().value("totalItems").toInt(),
+                        commentMap.value("likes").toMap().value("items").toList());
 
     // The comment itself
-    QStringList commentImageList = MiscHelpers::htmlWithReplacedImages(commentMap["content"].toString());
-    QString commentContents = commentImageList.takeAt(0); // comment HTML with images replaced
+    this->commentOriginalText = commentMap.value("content").toString();
 
-    contentLabel = new QLabel(commentContents);
-    contentLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
+
+    contentLabel = new QLabel();
+    contentLabel->setAlignment(Qt::AlignLeft);
     contentLabel->setFont(commentsFont);
     contentLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
     contentLabel->setWordWrap(true);
     contentLabel->setOpenExternalLinks(true);
     contentLabel->setTextFormat(Qt::RichText);
-    contentLabel->setSizePolicy(QSizePolicy::Minimum,
-                                QSizePolicy::MinimumExpanding);
+    contentLabel->setSizePolicy(sizePolicy);
     connect(contentLabel, SIGNAL(linkHovered(QString)),
             this, SLOT(showUrlInfo(QString)));
 
 
+
+    QStringList commentImageList = MiscHelpers::htmlWithReplacedImages(commentOriginalText,
+                                                                       64); // Arbitrary width
+    commentImageList.removeFirst(); // First is the HTML with images replaced
+
     // If the image list is not empty, get them
     if (!commentImageList.isEmpty())
     {
@@ -153,15 +178,7 @@ Comment::Comment(PumpController *pumpController,
 
         foreach (QString imageUrl, commentImageList)
         {
-            if (QFile::exists(MiscHelpers::getCachedImageFilename(imageUrl)))
-            {
-                qDebug() << "Image in comment already cached, ignoring";
-            }
-            else
-            {
-                qDebug() << "Image in comment not cached, downloading";
-                this->pController->getImage(imageUrl);
-            }
+            this->enqueueImageForDownload(imageUrl);
         }
     }
 
@@ -181,6 +198,7 @@ Comment::Comment(PumpController *pumpController,
 
 
     leftLayout = new QVBoxLayout();
+    leftLayout->setAlignment(Qt::AlignLeft);
     leftLayout->addWidget(avatarLabel,     0, Qt::AlignLeft | Qt::AlignTop);
     leftLayout->addWidget(likesCountLabel, 1, Qt::AlignHCenter | Qt::AlignTop);
 
@@ -188,6 +206,7 @@ Comment::Comment(PumpController *pumpController,
     rightTopLayout->addWidget(fullNameLabel,  0, Qt::AlignLeft);
     rightTopLayout->addWidget(timestampLabel, 0, Qt::AlignLeft);
     rightTopLayout->addWidget(likeLabel,      0, Qt::AlignLeft);
+    rightTopLayout->addWidget(quoteLabel,     0, Qt::AlignLeft);
     rightTopLayout->addStretch(1);
     if (commentIsOwn)
     {
@@ -196,21 +215,27 @@ Comment::Comment(PumpController *pumpController,
 
     rightLayout = new QVBoxLayout();
     rightLayout->addLayout(rightTopLayout, 0);
-    rightLayout->addSpacing(4); // 4px space
-    rightLayout->addWidget(contentLabel,   2);    
+    rightLayout->addSpacing(4); // 4px vertical space separation
+    rightLayout->addWidget(contentLabel,   10);
+
 
     mainLayout = new QHBoxLayout();
-    mainLayout->addLayout(leftLayout,  1);
-    mainLayout->addLayout(rightLayout, 15);
+    mainLayout->setAlignment(Qt::AlignTop);
+    mainLayout->addLayout(leftLayout);
+    mainLayout->addLayout(rightLayout);
     this->setLayout(mainLayout);
 
-    qDebug() << "Comment created";
+
+    setCommentContents();
+
+
+    qDebug() << "Comment created" << this->commentID;
 }
 
 
 Comment::~Comment()
 {
-    qDebug() << "Comment destroyed";
+    qDebug() << "Comment destroyed" << this->commentID;
 }
 
 
@@ -235,7 +260,7 @@ void Comment::setLikesCount(int count, QVariantList namesVariantList)
         QString likesString;
         foreach (QVariant likesMap, namesVariantList)
         {
-            likesString.append(likesMap.toMap()["displayName"].toString() + ", ");
+            likesString.append(likesMap.toMap().value("displayName").toString() + ", ");
         }
         likesString.remove(-2, 2); // remove last comma+space: ", "
         if (count > 1)
@@ -263,6 +288,66 @@ void Comment::setLikesCount(int count, QVariantList namesVariantList)
 }
 
 
+/*
+ * Set the contents of the comment, parsing images, etc.
+ *
+ */
+void Comment::setCommentContents()
+{
+    int imageWidth = this->contentLabel->width() - 20; // Kinda TMP
+
+    QStringList commentImageList = MiscHelpers::htmlWithReplacedImages(commentOriginalText,
+                                                                       imageWidth);
+
+    QString commentContents = commentImageList.takeAt(0); // Comment's HTML with images replaced
+    this->contentLabel->setText(commentContents);
+
+    setCommentHeight();
+}
+
+
+
+void Comment::setCommentHeight()
+{
+//    int height = this->contentLabel->height();
+
+/*
+    if (height > 200)
+    {
+        height = 200;
+    }
+*/
+
+/*
+    this->contentLabel->setMinimumHeight(height);
+    this->contentLabel->setMaximumHeight(height);
+*/
+
+}
+
+
+void Comment::enqueueImageForDownload(QString url)
+{
+    if (QFile::exists(MiscHelpers::getCachedImageFilename(url))
+     || pendingImagesList.contains(url))
+    {
+        qDebug() << "Comment::enqueueImageForDownload(), "
+                    "Using cached post image, or requested image is pending download...";
+    }
+    else
+    {
+        qDebug() << "Comment::enqueueImageForDownload(), "
+                    "post image not cached, downloading" << url;
+
+        connect(pController, SIGNAL(imageStored(QString)),
+                this, SLOT(redrawImages(QString)));
+
+        this->pendingImagesList.append(url);
+        pController->getImage(url);
+    }
+}
+
+
 
 /*
  * Ensure urlInfoLabel is hidden when the mouse leaves the comment
@@ -278,6 +363,7 @@ void Comment::leaveEvent(QEvent *event)
 
 
 
+
 /****************************************************************************/
 /******************************** SLOTS *************************************/
 /****************************************************************************/
@@ -303,6 +389,24 @@ void Comment::likeComment(QString clickedLink)
 }
 
 
+/*
+ * Take the contents of a comment and put them, as a quote block
+ * in the comment composer
+ *
+ * FIXME: It'd be nice to only quote the selected text,
+ * unless nothing is selected, then use full comment, but clicking
+ * the "quote" label removes the selection
+ */
+void Comment::quoteComment()
+{
+    QString quotedComment = MiscHelpers::quotedText(this->fullNameLabel->text(),
+                                                    this->contentLabel->text());
+
+    emit commentQuoteRequested(quotedComment);
+}
+
+
+
 void Comment::deleteComment()
 {
     int confirmation = QMessageBox::question(this, tr("WARNING: Delete comment?"),
@@ -341,3 +445,40 @@ void Comment::showUrlInfo(QString url)
         this->urlInfoLabel->show();
     }
 }
+
+
+
+/*
+ * Redraw comment author's avatar after receiving it
+ *
+ */void Comment::redrawAvatar(QString avatarUrl, QString avatarFilename)
+{
+    if (avatarUrl == this->commentAuthorAvatarUrl)
+    {
+        avatarLabel->setPixmap(QPixmap(avatarFilename)
+                               .scaledToHeight(32, Qt::SmoothTransformation));
+
+        disconnect(pController, SIGNAL(avatarStored(QString,QString)),
+                   this, SLOT(redrawAvatar(QString,QString)));
+    }
+}
+
+
+/*
+ * Redraw comment contents after receiving downloaded images
+ *
+ */
+void Comment::redrawImages(QString imageUrl)
+{
+    if (pendingImagesList.contains(imageUrl))
+    {
+        this->pendingImagesList.removeAll(imageUrl);
+        if (pendingImagesList.isEmpty()) // If there are no more, disconnect
+        {
+            disconnect(pController, SIGNAL(imageStored(QString)),
+                       this, SLOT(redrawImages(QString)));
+        }
+
+        setCommentContents();
+    }
+}
