@@ -29,6 +29,7 @@
 #include <QVariantMap>
 #include <QMessageBox>
 #include <QEvent>
+#include <QResizeEvent>
 
 #include <QDebug>
 
@@ -50,18 +51,29 @@ public:
     void fixLikeLabelText();
     void setLikesCount(int count, QVariantList namesVariantList);
 
+    void setCommentContents();
+    void setCommentHeight();
+    void enqueueImageForDownload(QString url);
+
+
 signals:
+    void commentQuoteRequested(QString content);
 
 
 public slots:
     void likeComment(QString clickedLink);
+    void quoteComment();
     void deleteComment();
 
     void showUrlInfo(QString url);
 
+    void redrawAvatar(QString avatarUrl, QString avatarFilename);
+    void redrawImages(QString imageUrl);
+
 protected:
     virtual void leaveEvent(QEvent *event);
 
+
 private:
     QHBoxLayout *mainLayout;
     QVBoxLayout *leftLayout;
@@ -75,6 +87,7 @@ private:
     QLabel *likesCountLabel;
 
     QLabel *likeLabel;
+    QLabel *quoteLabel;
     QLabel *deleteLabel;
 
     QHBoxLayout *contentLabelLayout;
@@ -82,8 +95,14 @@ private:
 
     QString commentID;
     QString objectType;
+    QString commentAuthorAvatarUrl;
+
     bool commentIsOwn;
     bool commentIsLiked;
+    QString commentOriginalText;
+    QStringList pendingImagesList;
+
+
     PumpController *pController;
 };
 
