@@ -20,7 +20,8 @@
 
 #include "commenterblock.h"
 
-CommenterBlock::CommenterBlock(PumpController *pumpController, QWidget *parent) : QWidget(parent)
+CommenterBlock::CommenterBlock(PumpController *pumpController,
+                               QWidget *parent) : QWidget(parent)
 {
     this->pController = pumpController;
 
@@ -37,7 +38,17 @@ CommenterBlock::CommenterBlock(PumpController *pumpController, QWidget *parent)
 
 
     commentsLayout = new QVBoxLayout();
+    commentsLayout->setAlignment(Qt::AlignLeft);
+
     commentsWidget = new QWidget();
+    QSizePolicy sizePolicy;
+    sizePolicy.setHeightForWidth(false);
+    sizePolicy.setWidthForHeight(false);
+    //sizePolicy.setHorizontalStretch(1);
+    //sizePolicy.setVerticalStretch(10);
+    sizePolicy.setHorizontalPolicy(QSizePolicy::MinimumExpanding);
+    sizePolicy.setVerticalPolicy(QSizePolicy::MinimumExpanding);
+    commentsWidget->setSizePolicy(sizePolicy);
     commentsWidget->setLayout(commentsLayout);
 
     this->commentsScrollArea = new QScrollArea();
@@ -76,10 +87,15 @@ CommenterBlock::CommenterBlock(PumpController *pumpController, QWidget *parent)
 
     this->commentButton = new QPushButton(QIcon::fromTheme("mail-message-new"),
                                           tr("&Comment"));
+    commentButton->setToolTip(tr("You can press Control+Enter to send "
+                                 "the comment with the keyboard") + "<b></b>");
+
     connect(commentButton, SIGNAL(clicked()), this, SLOT(sendComment()));
 
     this->cancelButton = new QPushButton(QIcon::fromTheme("dialog-cancel"),
                                          tr("C&ancel"));
+    cancelButton->setToolTip(tr("Press ESC to cancel the comment "
+                                "if there is no text") + "<b></b>");
     connect(cancelButton, SIGNAL(clicked()), commentComposer, SLOT(cancelPost()));
 
 
@@ -124,33 +140,67 @@ void CommenterBlock::setComments(QVariantList commentsList)
 
     if (commentsList.size() > 0)
     {
+        this->showAllCommentsLinkLabel->show();
+
+
         foreach (QVariant commentVariant, commentsList)
         {
             Comment *comment = new Comment(this->pController,
                                            commentVariant.toMap(),
                                            this);
             this->commentsInBlock.append(comment);
+            connect(comment, SIGNAL(commentQuoteRequested(QString)),
+                    this, SLOT(quoteComment(QString)));
 
             // Add the comment on top, since the list comes in reverse
             this->commentsLayout->insertWidget(0, comment);
         }
 
 
-        this->showAllCommentsLinkLabel->show();
         this->commentsScrollArea->show();
 
+
         // Move scrollbar to the bottom
         this->scrollCommentsToBottom(); // First, try
+
         // Then, 50 msec later, try again, in case there was no scrollbar before,
         // and the first try didn't work
         // Trying immediately first avoids a flicker-like effect sometimes
         scrollToBottomTimer->start(50);
+
+        // Trigger resizeEvent
+        //this->commentsWidget->resize(commentsWidget->width() - 1, commentsWidget->height() -1);
     }
 }
 
 
 
 
+void CommenterBlock::resizeEvent(QResizeEvent *event)
+{
+/*
+    int commentWidth = this->commentsScrollArea->width();
+    if (commentsScrollArea->verticalScrollBar()->isVisible())
+    {
+        // account for the width of the scrollbar
+        commentWidth -= 32;  // FIXME, don't hardcode it
+    }
+
+    commentsWidget->setMaximumWidth(commentWidth);
+*/
+
+
+    foreach (Comment *comment, this->commentsInBlock)
+    {
+        comment->setCommentContents();
+    }
+
+    event->accept();
+}
+
+
+
+
 /*******************************************************************/
 /****************************** SLOTS ******************************/
 /*******************************************************************/
@@ -174,7 +224,7 @@ void CommenterBlock::setMinimumMode()
 
 
 
-void CommenterBlock::setFullMode()
+void CommenterBlock::setFullMode(QString initialText)
 {
     this->commentComposer->show();
     this->statusInfoLabel->show();
@@ -182,6 +232,7 @@ void CommenterBlock::setFullMode()
     this->cancelButton->show();
 
     this->commentComposer->setFocus();
+    this->commentComposer->append(initialText);
 
     // Block updates to timeline while we're commenting...
     this->pController->setUpdatesToTimelineBlocked(true);
@@ -191,6 +242,13 @@ void CommenterBlock::setFullMode()
 }
 
 
+void CommenterBlock::quoteComment(QString content)
+{
+    this->setFullMode();
+    this->commentComposer->append(content);
+}
+
+
 void CommenterBlock::requestAllComments()
 {
     emit allCommentsRequested();
@@ -234,7 +292,7 @@ void CommenterBlock::onPostingCommentFailed()
     qDebug() << "Posting the comment failed, re-enabling Commenter";
 
     // Alert about the error
-    this->statusInfoLabel->setText(tr("Posting comment failed.\nTry again."));
+    this->statusInfoLabel->setText(tr("Posting comment failed.\n\nTry again."));
 
     // Re-enable things, so user can try again
     this->setEnabled(true);
@@ -266,6 +324,7 @@ void CommenterBlock::sendComment()
     }
     else
     {
+        this->statusInfoLabel->setText(tr("Comment is empty."));
         qDebug() << "Can't post, comment is empty";
     }
 }
