@@ -44,7 +44,7 @@ public:
 
     void clearComments();
     void setComments(QVariantList commentsList);
-    
+
 signals:
     void commentSent(QString comment);
     void allCommentsRequested();
@@ -52,7 +52,9 @@ signals:
 
 public slots:
     void setMinimumMode();
-    void setFullMode();
+    void setFullMode(QString initialText="");
+
+    void quoteComment(QString content);
 
     void requestAllComments();
     void onPostingCommentOk();
@@ -62,6 +64,10 @@ public slots:
 
     void scrollCommentsToBottom();
 
+
+protected:
+    virtual void resizeEvent(QResizeEvent *event);
+
 private:
     QGridLayout *mainLayout;
 
