@@ -28,8 +28,14 @@ Composer::Composer(bool forPublisher)
     this->setAcceptRichText(true);
     this->setTabChangesFocus(true);
 
+    // FIXME: do something about the links being styled with user's color
+    //this->setStyleSheet("{ a:link { color: #FF0000;} }");
+
+    QFont startConversationFont;
+    startConversationFont.setPointSize(startConversationFont.pointSize() - 2);
 
     startConversationLabel = new QLabel(tr("Click here or press Control+N to post a note..."));
+    startConversationLabel->setFont(startConversationFont);
 
     // A menu to insert some Unicode symbols
     symbolsMenu = new QMenu(tr("Symbols"));
@@ -67,13 +73,26 @@ Composer::Composer(bool forPublisher)
                          this,
                          SLOT(makeUnderline()),
                          QKeySequence::Underline);
+    toolsMenu->addAction(QIcon::fromTheme("format-text-strikethrough"),
+                         tr("Strikethrough"),
+                         this,
+                         SLOT(makeStrikethrough()));
 
     toolsMenu->addSeparator();
 
+    toolsMenu->addAction(QIcon::fromTheme("format-font-size-more"),
+                         tr("Header"),
+                         this,
+                         SLOT(makeHeader()),
+                         QKeySequence("Ctrl+H"));
     toolsMenu->addAction(QIcon::fromTheme("format-justify-fill"),
                          tr("Preformatted block"),
                          this,
                          SLOT(makePreformatted()));
+    toolsMenu->addAction(QIcon::fromTheme("format-text-italic"),
+                         tr("Quote block"),
+                         this,
+                         SLOT(makeQuote()));
 
     toolsMenu->addSeparator();
 
@@ -86,6 +105,10 @@ Composer::Composer(bool forPublisher)
                          tr("Insert an image from a web site"),
                          this,
                          SLOT(insertImage()));
+    toolsMenu->addAction(QIcon::fromTheme("insert-horizontal-rule"),
+                         tr("Insert line"),
+                         this,
+                         SLOT(insertLine()));
 
     toolsMenu->addSeparator();
 
@@ -98,14 +121,25 @@ Composer::Composer(bool forPublisher)
                          this,
                          SLOT(cancelPost()));
 
-    toolsButton = new QPushButton(QIcon::fromTheme("format-list-ordered"), tr("&F", "F for formatting, translate accordingly"));
+    toolsButton = new QPushButton(QIcon::fromTheme("format-list-ordered"),
+                                  tr("&F", "F for formatting, translate accordingly"));
     toolsButton->setMenu(toolsMenu);
     toolsButton->setToolTip(tr("Text Formatting Options"));
     toolsButton->setHidden(true); // Hidden on startup (Publisher in miniMode)
 
 
-    this->customContextMenu = this->createStandardContextMenu();
-    this->customContextMenu->addMenu(toolsMenu);
+    // Extra action for the context menu, paste as plaintext
+    this->pastePlaintextAction = new QAction(QIcon::fromTheme("paste"),
+                                             tr("Paste Text Without Formatting"),
+                                             this);
+
+    pastePlaintextAction->setShortcut(QKeySequence("Ctrl+Shift+V")); // This doesn't work like this
+    connect(pastePlaintextAction, SIGNAL(triggered()),
+            this, SLOT(pasteAsPlaintext()));
+
+    // Add action to this object, in order for the shortcut to work
+    // Otherwise, the action isn't available until the context menu is present
+    this->addAction(pastePlaintextAction);
 
     mainLayout = new QGridLayout();
 
@@ -118,9 +152,7 @@ Composer::Composer(bool forPublisher)
     }
     else                     // or Commenter mode
     {
-        this->setToolTip(tr("Type a comment here.\n"
-                            "You can press Control+Enter to send the comment\n"
-                            "with the keyboard, or ESC to cancel."));
+        this->setToolTip(tr("Type a comment here"));
     }
 
     mainLayout->addWidget(toolsButton,            0, 1, 1, 1, Qt::AlignRight | Qt::AlignBottom);
@@ -150,6 +182,19 @@ void Composer::erase()
 }
 
 
+void Composer::insertLink(QString url, QString title)
+{
+    if (title.isEmpty())
+    {
+        title = url;
+    }
+
+    this->insertHtml("<a href=\"" + url + "\">"
+                     "<span style=\"color:#0088CC\">"
+                     + title + "</span></a> "); // Important, space after!
+}
+
+
 /*
  * Send a signal when getting focus
  *
@@ -206,7 +251,53 @@ void Composer::keyPressEvent(QKeyEvent *event)
  */
 void Composer::contextMenuEvent(QContextMenuEvent *event)
 {
+    this->customContextMenu = this->createStandardContextMenu();
+
+    // tools menu before default context menu
+    customContextMenu->insertMenu(customContextMenu->actions().at(0),
+                                  toolsMenu);
+    customContextMenu->insertSeparator(customContextMenu->actions().at(1));
+
+    // And options added after default context menu
+    customContextMenu->addSeparator();
+
+
+    customContextMenu->addAction(pastePlaintextAction);
+    if (this->canPaste())
+    {
+        pastePlaintextAction->setEnabled(true);
+    }
+    else
+    {
+        pastePlaintextAction->setDisabled(true);
+    }
+
     customContextMenu->exec(event->globalPos());
+
+    // FIXME: Possible leak... should delete customContextMenu?
+    customContextMenu->deleteLater();
+
+    event->accept();
+}
+
+
+/*
+ * Intervene when pasting, so we can turn links into real HTML links
+ *
+ */
+void Composer::insertFromMimeData(const QMimeData *source)
+{
+    QString pastedText = source->text().trimmed();
+
+    if (pastedText.startsWith("http://")
+     || pastedText.startsWith("https://"))
+    {
+        this->insertLink(pastedText);
+    }
+    else // Just paste original contents
+    {
+        QTextEdit::insertFromMimeData(source);
+    }
 }
 
 
@@ -223,7 +314,12 @@ void Composer::contextMenuEvent(QContextMenuEvent *event)
  */
 void Composer::makeNormal()
 {
-    this->setCurrentCharFormat(QTextCharFormat());
+    QTextCharFormat charFormat;
+    charFormat.clearForeground();
+    charFormat.clearBackground();
+    this->setCurrentCharFormat(charFormat);
+
+    this->setFocus();
 }
 
 
@@ -278,6 +374,36 @@ void Composer::makeUnderline()
     this->setFocus();
 }
 
+/*
+ * Strike out selected text
+ *
+ */
+void Composer::makeStrikethrough()
+{
+    QTextCharFormat charFormat;
+    charFormat.setFontStrikeOut(!this->currentCharFormat().fontStrikeOut());
+    this->mergeCurrentCharFormat(charFormat);
+
+    this->setFocus();
+}
+
+/*
+ * Turn the selected text into an <h2> header
+ *
+ */
+void Composer::makeHeader()
+{
+    QString selectedText = this->textCursor().selectedText();
+
+    if (!selectedText.isEmpty())
+    {
+        this->textCursor().removeSelectedText();
+        this->insertHtml("<h2>" + selectedText + "</h2> ");
+    }
+
+    this->setFocus();
+}
+
 
 
 /*
@@ -297,6 +423,26 @@ void Composer::makePreformatted()
     this->setFocus();
 }
 
+/*
+ * Mark selected as quoted, using <blockquote>
+ *
+ */
+void Composer::makeQuote()
+{
+    QString selectedText = this->textCursor().selectedText();
+
+    if (!selectedText.isEmpty())
+    {
+        this->textCursor().removeSelectedText();
+        this->insertHtml("<br />"
+                         "<blockquote>&ldquo;" + selectedText + "&rdquo;</blockquote> ");
+    }
+    // FIXME: Qt's HTML changes <blockquote> into its own formatting
+    // so other clients and the web UI might not display it as other people's blockquote's
+
+    this->setFocus();
+}
+
 
 
 
@@ -317,7 +463,6 @@ void Composer::makeLink()
                                      tr("Type or paste a web address here.\n"),
                                      QLineEdit::Normal,
                                      "http://");
-        selectedText = link;
     }
     else
     {
@@ -330,13 +475,12 @@ void Composer::makeLink()
     }
 
 
-
     if (!link.isEmpty())
     {
         link = link.trimmed(); // Remove possible spaces before or after
 
         this->textCursor().removeSelectedText();
-        this->insertHtml("<a href=\"" + link + "\">" + selectedText + "</a> ");
+        this->insertLink(link, selectedText);
     }
     else
     {
@@ -366,16 +510,39 @@ void Composer::insertImage()
 
     if (!imageURL.isEmpty())
     {
-        this->insertHtml("<img src=\"" + imageURL + "\" />");
+        if (imageURL.startsWith("http://")
+         || imageURL.startsWith("https://"))
+        {
+            this->insertHtml("<img src=\"" + imageURL + "\" />");
+        }
+        else
+        {
+            QMessageBox::warning(this, tr("Error: Invalid URL"),
+                                 tr("The address you entered (%1) "
+                                    "is not valid.\n"
+                                    "Image addresses should begin "
+                                    "with http:// or https://").arg(imageURL));
+        }
     }
     else
     {
-        qDebug() << "insertImage(): Invalid image URL";
+        qDebug() << "insertImage(): Image URL is empty";
     }
 
     this->setFocus();
 }
 
+/*
+ * Insert a horizontal line, <hr>
+ *
+ */
+void Composer::insertLine()
+{
+    this->insertHtml("<hr><br>");
+
+    this->setFocus();
+}
+
 
 
 void Composer::insertSymbol(QAction *action)
@@ -388,6 +555,18 @@ void Composer::insertSymbol(QAction *action)
 
 
 
+void Composer::pasteAsPlaintext()
+{
+    QString subtype("plain");
+    QString clipboardContents = QApplication::clipboard()->text(subtype,
+                                                                QClipboard::Clipboard);
+
+    this->makeNormal(); // Ensure normal text before insterting
+    this->insertPlainText(clipboardContents);
+}
+
+
+
 
 /*
  * Cancel editing of the post, clear it, return to minimum mode
