@@ -29,6 +29,9 @@
 #include <QMenu>
 #include <QInputDialog>
 #include <QMessageBox>
+#include <QApplication>
+#include <QClipboard>
+#include <QMimeData> // Needed in Qt5
 
 #include <QDebug>
 
@@ -41,6 +44,7 @@ public:
     ~Composer();
 
     void erase();
+    void insertLink(QString url, QString title="");
 
 signals:
     void focusReceived();
@@ -52,12 +56,18 @@ public slots:
     void makeBold();
     void makeItalic();
     void makeUnderline();
+    void makeStrikethrough();
+    void makeHeader();
     void makePreformatted();
+    void makeQuote();
     void makeLink();
     void insertImage();
+    void insertLine();
 
     void insertSymbol(QAction *action);
 
+    void pasteAsPlaintext();
+
     void cancelPost();
 
 
@@ -65,6 +75,7 @@ protected:
     virtual void focusInEvent(QFocusEvent *event);
     virtual void keyPressEvent(QKeyEvent *event);
     virtual void contextMenuEvent(QContextMenuEvent *event);
+    virtual void insertFromMimeData(const QMimeData *source);
 
 private:
     QGridLayout *mainLayout;
@@ -74,6 +85,8 @@ private:
     QMenu *toolsMenu;
     QMenu *symbolsMenu;
 
+    QAction *pastePlaintextAction;
+
     QMenu *customContextMenu;
 
 
