@@ -39,7 +39,8 @@ class ConfigDialog : public QWidget
     Q_OBJECT
 
 public:
-    ConfigDialog(int updateInterval,
+    ConfigDialog(QString dataDirectory,
+                 int updateInterval,
                  int postsPerPage,
                  int tabsPosition,
                  bool tabsMovable,
@@ -77,6 +78,7 @@ private:
     QCheckBox *publicPostsCheckbox;
     QComboBox *showNotificationsCombobox;
 
+    QLabel *dataDirectoryLabel;
 
     QHBoxLayout *buttonsLayout;
     QPushButton *saveConfigButton;
