@@ -31,6 +31,8 @@
 #include <QFont>
 #include <QFile>
 #include <QPushButton>
+#include <QMenu>
+#include <QMessageBox>
 
 #include <QDebug>
 
@@ -57,19 +59,32 @@ public slots:
     void followContact();
     void unfollowContact();
 
+    void openProfileInBrowser();
+
+    void redrawAvatar(QString avatarUrl, QString avatarFilename);
+
+
 private:
     QHBoxLayout *mainLayout;
     QVBoxLayout *centerLayout;
+    QVBoxLayout *rightLayout;
 
     QLabel *avatarLabel;
     QLabel *nameLabel;
 
-    QLabel *tmpLabel;
+    QLabel *userInfoLabel;
 
     QPushButton *followButton;
+    QPushButton *optionsButton;
+    QMenu *optionsMenu;
+    QAction *openProfileAction;
+    QMenu *addToListMenu;
+
 
     PumpController *pController;
     QString contactID;
+    QString contactURL;
+    QString contactAvatarUrl;
 };
 
 #endif // CONTACTCARD_H
