@@ -29,18 +29,25 @@ ContactList::ContactList(PumpController *pumpController,
     mainLayout->setAlignment(Qt::AlignTop);
 
 
-    QString webfingerHelpMessage = tr("Enter a Webfinger address, like username@server.org "
-                                      "or https://server.org/username");
+    QString webfingerHelpMessage = tr("username@server.org or https://server.org/username");
 
     topLayout = new QHBoxLayout();
+    enterAddressLabel = new QLabel(tr("&Enter address to follow:"));
+    enterAddressLabel->setToolTip(webfingerHelpMessage);
+
     addressLineEdit = new QLineEdit();
     addressLineEdit->setPlaceholderText(webfingerHelpMessage);
     addressLineEdit->setToolTip(webfingerHelpMessage);
-    this->addressLineEdit->setFocusPolicy(Qt::StrongFocus); // Prevents initial focus, somehow
+    connect(addressLineEdit, SIGNAL(returnPressed()),
+            this, SLOT(followContact()));
+
+    enterAddressLabel->setBuddy(addressLineEdit);
+
 
     followButton = new QPushButton(QIcon::fromTheme("list-add-user"), tr("&Follow"));
     connect(followButton, SIGNAL(clicked()),
             this, SLOT(followContact()));
+    topLayout->addWidget(enterAddressLabel);
     topLayout->addWidget(addressLineEdit);
     topLayout->addWidget(followButton);
 
@@ -87,36 +94,39 @@ ContactList::ContactList(PumpController *pumpController,
 
 
     // Widget for the list of 'lists'
-    this->listsLabel = new QLabel();
-    listsLabel->setAlignment(Qt::AlignHCenter | Qt::AlignTop);
-
+    this->listsManager = new ListsManager(this->pController);
     listsScrollArea = new QScrollArea();
-    listsScrollArea->setWidget(listsLabel);
+    listsScrollArea->setWidget(this->listsManager);
     listsScrollArea->setWidgetResizable(true);
 
 
     // Options menu
-    optionsMenu = new QMenu("options-menu");
-    optionsMenu->addAction(QIcon::fromTheme("view-refresh"),
-                           tr("Reload Following"),
-                           this,
-                           SLOT(refreshFollowing()));
+    optionsMenu = new QMenu("*options-menu*");
     optionsMenu->addAction(QIcon::fromTheme("view-refresh"),
                            tr("Reload Followers"),
                            this,
                            SLOT(refreshFollowers()));
-    optionsMenu->addSeparator();
-    optionsMenu->addAction(QIcon::fromTheme("document-export"),
-                           tr("Export Following"),
+    optionsMenu->addAction(QIcon::fromTheme("view-refresh"),
+                           tr("Reload Following"),
                            this,
-                           SLOT(exportFollowing()));
+                           SLOT(refreshFollowing()));
+    optionsMenu->addSeparator();
     optionsMenu->addAction(QIcon::fromTheme("document-export"),
                            tr("Export Followers"),
                            this,
                            SLOT(exportFollowers()));
+    optionsMenu->addAction(QIcon::fromTheme("document-export"),
+                           tr("Export Following"),
+                           this,
+                           SLOT(exportFollowing()));
+    optionsMenu->addSeparator();
+    optionsMenu->addAction(QIcon::fromTheme("view-refresh"),
+                           tr("Reload Lists"),
+                           this,
+                           SLOT(refreshLists()));
 
 
-    optionsButton = new QPushButton(QIcon::fromTheme("user-properties"),
+    optionsButton = new QPushButton(QIcon::fromTheme("configure"),
                                     tr("Optio&ns"));
     optionsButton->setMenu(optionsMenu);
 
@@ -124,12 +134,12 @@ ContactList::ContactList(PumpController *pumpController,
     this->tabWidget = new QTabWidget();
     tabWidget->setDocumentMode(true); // To have less margins, with so many tabs inside tabs...
 
-    tabWidget->addTab(followingScrollArea,
-                      QIcon::fromTheme("meeting-participant"),
-                      "following");
     tabWidget->addTab(followersScrollArea,
                       QIcon::fromTheme("meeting-observer"),
                       "followers");
+    tabWidget->addTab(followingScrollArea,
+                      QIcon::fromTheme("meeting-participant"),
+                      "following");
     tabWidget->addTab(listsScrollArea,
                       QIcon::fromTheme("preferences-contact-list"),
                       "lists");
@@ -137,8 +147,8 @@ ContactList::ContactList(PumpController *pumpController,
     tabWidget->setCornerWidget(this->optionsButton);
 
 
-    this->followingCount = 0;
     this->followersCount = 0;
+    this->followingCount = 0;
     this->listsCount = 0;
     this->setTabLabels();
 
@@ -146,7 +156,7 @@ ContactList::ContactList(PumpController *pumpController,
     mainLayout->addWidget(tabWidget);
     this->setLayout(mainLayout);
 
-
+    // FIXME: this doesn't really work
     this->followButton->setFocus(); // Set focus here, away from addressLineEdit
 
     qDebug() << "Contact list created";
@@ -189,12 +199,12 @@ void ContactList::clearFollowersListContents()
 
 void ContactList::setTabLabels()
 {
-    this->tabWidget->setTabText(0, tr("Followin&g")
-                                   + QString(" (%1)").arg(this->followingCount));
-
-    this->tabWidget->setTabText(1, tr("Follo&wers")
+    this->tabWidget->setTabText(0, tr("Follo&wers")
                                    + QString(" (%1)").arg(this->followersCount));
 
+    this->tabWidget->setTabText(1, tr("Followin&g")
+                                   + QString(" (%1)").arg(this->followingCount));
+
     this->tabWidget->setTabText(2, tr("&Lists")
                                    + QString(" (%1)").arg(this->listsCount));
 }
@@ -271,22 +281,25 @@ void ContactList::setContactListContents(QString listType, QVariantList contactL
 
     QMap<QString,QString> contactData;
     QString contactInfoLineString;
+    QStringList followingIdStringList;
 
     foreach (QVariant contact, contactList)
     {
         QVariantMap contactMap = contact.toMap();
 
-        contactData.insert("id", contactMap["id"].toString().remove(0,5)); // remove "acct:"
-        contactData.insert("name", contactMap["displayName"].toString());
-        contactData.insert("avatar", contactMap["image"].toMap().value("url").toString());
-        contactData.insert("hometown", contactMap["location"].toMap().value("displayName").toString());
-        contactData.insert("bio", contactMap["summary"].toString());
+        contactData.insert("id", contactMap.value("id").toString().remove(0,5)); // remove "acct:"
+        contactData.insert("name", contactMap.value("displayName").toString());
+        contactData.insert("avatar", contactMap.value("image").toMap().value("url").toString());
+        contactData.insert("hometown", contactMap.value("location").toMap().value("displayName").toString());
+        contactData.insert("bio", contactMap.value("summary").toString());
 
-        contactData.insert("following", contactMap["pump_io"].toMap().value("followed").toString());
+        contactData.insert("url", contactMap.value("url").toString());
 
-        contactInfoLineString = contactData["name"]
+        contactData.insert("following", contactMap.value("pump_io").toMap().value("followed").toString());
+
+        contactInfoLineString = contactData.value("name")
                               + "  <"
-                              + contactData["id"]
+                              + contactData.value("id")
                               + ">\n";
 
         if (listType == "following")
@@ -296,6 +309,7 @@ void ContactList::setContactListContents(QString listType, QVariantList contactL
                                                             this));
             followingListString.append(contactInfoLineString);
 
+            followingIdStringList.append(contactData.value("id"));
         }
         else // == "followers"
         {
@@ -306,6 +320,11 @@ void ContactList::setContactListContents(QString listType, QVariantList contactL
         }
     }
 
+    if (listType == "following")
+    {
+        this->pController->updateInternalFollowingIdList(followingIdStringList);
+    }
+
 
     // Update tab labels with number of following or followers, which were updated before
     this->setTabLabels();
@@ -319,38 +338,12 @@ void ContactList::setContactListContents(QString listType, QVariantList contactL
  */
 void ContactList::setListsListContents(QVariantList listsList)
 {
-    qDebug() << "Setting lists list contents";
-
-    QString listName;
-    int listMembersCount;
-
-    QString listsString = "<br>"
-                          "<table align=\"center\">"
-                          "<tr>"
-                          "<td>" + tr("Name") + "</td>"
-                          "<td>&nbsp;&nbsp;&nbsp;</td>"
-                          "<td align=\"right\">" + tr("Members") + "<br><br></td>"
-                          "</tr>";
-    foreach (QVariant list, listsList)
-    {
-        listName = list.toMap()["displayName"].toString();
-        listMembersCount = list.toMap()["members"].toMap()["totalItems"].toInt();
-
-        listsString.append("<tr>");
-        listsString.append(QString("<td><b>%1</b></td>").arg(listName));
-        listsString.append("<td></td>");
-        listsString.append(QString("<td align=\"right\">%1<br></td>").arg(listMembersCount));
-        listsString.append("</tr>");
-    }
-    listsString.append("</table>");
-
-
-    this->listsLabel->setText(listsString);
-
-
     this->listsCount = listsList.count();
     // Update tab labels with number of following or followers, which were updated before
     this->setTabLabels();
+
+
+    this->listsManager->setListsList(listsList);
 }
 
 
@@ -399,6 +392,13 @@ void ContactList::exportFollowers()
 }
 
 
+void ContactList::refreshLists()
+{
+    qDebug() << "Refreshing list of lists...";
+    this->pController->getListsList();
+}
+
+
 
 
 /*
@@ -420,9 +420,10 @@ void ContactList::followContact()
     {
         validAddress = true;
     }
-    else if (address.startsWith("https://"))
+    else if (address.startsWith("https://") || address.startsWith("http://"))
     {
         address.remove("https://");
+        address.remove("http://");
 
         if (address.contains("/")) // Very basic sanity check
         {
