@@ -39,6 +39,7 @@
 
 #include "pumpcontroller.h"
 #include "contactcard.h"
+#include "listsmanager.h"
 
 
 class ContactList : public QWidget
@@ -69,12 +70,15 @@ public slots:
     void exportFollowing();
     void exportFollowers();
 
+    void refreshLists();
+
     void followContact();
 
 private:
     QVBoxLayout *mainLayout;
 
     QHBoxLayout *topLayout;
+    QLabel *enterAddressLabel;
     QLineEdit *addressLineEdit;
     QPushButton *followButton;
 
@@ -94,12 +98,11 @@ private:
     QString followersListString;
 
 
-    QLabel *listsLabel;
+    ListsManager *listsManager;
     QScrollArea *listsScrollArea;
     int listsCount;
 
 
-
     QPushButton *optionsButton;
     QMenu *optionsMenu;
 
