@@ -30,7 +30,7 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     QSettings settings;
 
     firstRun = true;
-    prepareDataDirectory();
+    prepareDataDirectory(); // This sets this->dataDirectory
 
     reallyQuitProgram = false;
     trayIconAvailable = false;
@@ -50,19 +50,31 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
 
 
     ////// GUI
+
+    // Splitter to divide the window horizontally
     mainSplitter = new QSplitter(Qt::Horizontal, this);
 
+    // User's profile editor, in its own window
+    profileEditor = new ProfileEditor(pumpController, this);
+
 
     // Left side
     leftSideWidget = new QWidget();
     leftLayout = new QVBoxLayout();
 
-    avatarIconLabel = new QLabel();
-    avatarIconLabel->setPixmap(QPixmap(":/images/no-avatar.png")
-                             .scaled(64, 64,
-                                     Qt::KeepAspectRatio,
-                                     Qt::SmoothTransformation));
-    leftLayout->addWidget(avatarIconLabel);
+    avatarIconButton = new QPushButton();
+    avatarIconButton->setIcon(QIcon(QPixmap(":/images/no-avatar.png")
+                                    .scaled(64, 64,
+                                            Qt::KeepAspectRatio,
+                                            Qt::SmoothTransformation)));
+    avatarIconButton->setSizePolicy(QSizePolicy::Maximum,
+                                    QSizePolicy::Maximum);
+    avatarIconButton->setFlat(true);
+    avatarIconButton->setIconSize(QSize(64, 64));
+    connect(avatarIconButton, SIGNAL(clicked()),
+            profileEditor, SLOT(show()));
+
+    leftLayout->addWidget(avatarIconButton, 0, Qt::AlignLeft);
     leftLayout->addSpacing(4);
 
     fullNameLabel = new QLabel("[----------]");
@@ -111,12 +123,18 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     publisher = new Publisher(pumpController, publisherType);
 
 
+    /// START SETTING UP TIMELINES
+
+
+    // Main timeline //
+
     mainTimeline = new TimeLine(TimeLine::TimelineTypeMain,
                                 pumpController,
                                 this);
     mainTimelineScrollArea = new QScrollArea();
     mainTimelineScrollArea->setWidget(mainTimeline);  // Make timeline scrollable
     mainTimelineScrollArea->setWidgetResizable(true);
+    mainTimelineScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
     connect(mainTimeline, SIGNAL(scrollToTop()),
             this, SLOT(scrollMainTimelineToTop()));
     connect(mainTimeline, SIGNAL(timelineRendered(int,int)),
@@ -127,6 +145,16 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     connect(mainTimeline, SIGNAL(timelineRendered(int,int)),
             this, SLOT(notifyTimelineUpdate(int,int)));
 
+    // for post editing
+    connect(mainTimeline, SIGNAL(postEditRequested(QString,QString,QString)),
+            publisher, SLOT(setEditingMode(QString,QString,QString)));
+
+    // To ensure comment composer is visible
+    connect(mainTimeline, SIGNAL(commentingOnPost(QWidget*)),
+            this, SLOT(scrollMainTimelineToWidget(QWidget*)));
+
+
+    // Direct timeline //
 
     directTimeline = new TimeLine(TimeLine::TimelineTypeDirect,
                                   pumpController,
@@ -134,6 +162,7 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     directTimelineScrollArea = new QScrollArea();
     directTimelineScrollArea->setWidget(directTimeline);
     directTimelineScrollArea->setWidgetResizable(true);
+    directTimelineScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
     connect(directTimeline, SIGNAL(scrollToTop()),
             this, SLOT(scrollDirectTimelineToTop()));
     connect(directTimeline, SIGNAL(timelineRendered(int,int)),
@@ -144,6 +173,15 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     connect(directTimeline, SIGNAL(timelineRendered(int,int)),
             this, SLOT(notifyTimelineUpdate(int,int)));
 
+    // direct timeline doesn't need a connection for post editing,
+    // since you're not gonna send a message to yourself =)
+
+    // To ensure comment composer is visible
+    connect(directTimeline, SIGNAL(commentingOnPost(QWidget*)),
+            this, SLOT(scrollDirectTimelineToWidget(QWidget*)));
+
+
+    // Activity timeline //
 
     activityTimeline = new TimeLine(TimeLine::TimelineTypeActivity,
                                     pumpController,
@@ -151,6 +189,7 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     activityTimelineScrollArea = new QScrollArea();
     activityTimelineScrollArea->setWidget(activityTimeline);  // Make it scrollable
     activityTimelineScrollArea->setWidgetResizable(true);
+    activityTimelineScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
     connect(activityTimeline, SIGNAL(scrollToTop()),
             this, SLOT(scrollActivityTimelineToTop()));
     connect(activityTimeline, SIGNAL(timelineRendered(int,int)),
@@ -161,6 +200,16 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     connect(activityTimeline, SIGNAL(timelineRendered(int,int)),
             this, SLOT(notifyTimelineUpdate(int,int)));
 
+    // for post editing
+    connect(activityTimeline, SIGNAL(postEditRequested(QString,QString,QString)),
+            publisher, SLOT(setEditingMode(QString,QString,QString)));
+
+    // To ensure comment composer is visible
+    connect(activityTimeline, SIGNAL(commentingOnPost(QWidget*)),
+            this, SLOT(scrollActivityTimelineToWidget(QWidget*)));
+
+
+    // Favorites timeline //
 
     favoritesTimeline = new TimeLine(TimeLine::TimelineTypeFavorites,
                                      pumpController,
@@ -168,6 +217,7 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     favoritesTimelineScrollArea = new QScrollArea();
     favoritesTimelineScrollArea->setWidget(favoritesTimeline);
     favoritesTimelineScrollArea->setWidgetResizable(true);
+    favoritesTimelineScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
     connect(favoritesTimeline, SIGNAL(scrollToTop()),
             this, SLOT(scrollFavoritesTimelineToTop()));
     connect(favoritesTimeline, SIGNAL(timelineRendered(int,int)),
@@ -178,6 +228,17 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     connect(favoritesTimeline, SIGNAL(timelineRendered(int,int)),
             this, SLOT(notifyTimelineUpdate(int,int)));
 
+    // for post editing
+    connect(favoritesTimeline, SIGNAL(postEditRequested(QString,QString,QString)),
+            publisher, SLOT(setEditingMode(QString,QString,QString)));
+
+    // To ensure comment composer is visible
+    connect(favoritesTimeline, SIGNAL(commentingOnPost(QWidget*)),
+            this, SLOT(scrollFavoritesTimelineToWidget(QWidget*)));
+
+
+    /// END SETTING UP TIMELINES
+
 
     // The contact list has its own tabs with its own scroll areas
     contactList = new ContactList(pumpController, this);
@@ -246,14 +307,16 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     updateTimer->start();
 
 
-    profileEditor = new ProfileEditor(pumpController, this);
-
+    //// External widgets which live in their own windows
 
+    // Account wizard
     accountDialog = new AccountDialog(pumpController, this);
     connect(accountDialog, SIGNAL(userIDChanged(QString)),
             this, SLOT(updateUserID(QString)));
 
-    configDialog = new ConfigDialog(this->updateInterval,
+    // Configuration dialog
+    configDialog = new ConfigDialog(this->dataDirectory,
+                                    this->updateInterval,
                                     this->postsPerPage,
                                     this->tabsPosition,
                                     this->tabsMovable,
@@ -415,6 +478,15 @@ void MainWindow::closeEvent(QCloseEvent *event)
 }
 
 
+void MainWindow::resizeEvent(QResizeEvent *event)
+{
+    qDebug() << "MainWindow::resizeEvent()" << event->size();
+    this->adjustTimelineSizes();
+
+    event->accept();
+}
+
+
 
 
 /*
@@ -516,6 +588,7 @@ void MainWindow::createMenus()
     sessionUpdateMinorFeed = new QAction(QIcon::fromTheme("view-refresh"),
                                          tr("Update Minor &Feed"),
                                          this);
+    sessionUpdateMinorFeed->setShortcut(QKeySequence(Qt::Key_F10));
     connect(sessionUpdateMinorFeed, SIGNAL(triggered()),
             meanwhileFeed, SLOT(updateFeed()));
     sessionMenu->addAction(sessionUpdateMinorFeed);
@@ -531,6 +604,16 @@ void MainWindow::createMenus()
 
     sessionMenu->addSeparator();
 
+    sessionMarkAllAsRead = new QAction(QIcon::fromTheme("mail-mark-read"),
+                                       tr("Mark All as Read"),
+                                       this);
+    // connect(...
+    sessionMarkAllAsRead->setDisabled(true); // TMPFIX: until implemented
+    sessionMarkAllAsRead->setVisible(false); // Same thing!
+    sessionMenu->addAction(sessionMarkAllAsRead);    
+
+    sessionMenu->addSeparator();
+
     sessionPostNote = new QAction(QIcon::fromTheme("document-edit"),
                                           tr("&Post a Note"),
                                           this);
@@ -562,7 +645,7 @@ void MainWindow::createMenus()
             this, SLOT(toggleSidePanel(bool)));
     viewSidePanel->setCheckable(true);
     viewSidePanel->setChecked(true);
-    viewSidePanel->setShortcut(QKeySequence("F9"));
+    viewSidePanel->setShortcut(Qt::Key_F9);
 
     viewMenu->addAction(viewSidePanel);
 
@@ -575,6 +658,18 @@ void MainWindow::createMenus()
 
     viewMenu->addAction(viewStatusBar);
 
+
+    viewFullscreenAction = new QAction(QIcon::fromTheme("view-fullscreen"),
+                                       tr("Full &Screen"), this);
+    connect(viewFullscreenAction, SIGNAL(toggled(bool)),
+            this, SLOT(toggleFullscreen(bool)));
+    viewFullscreenAction->setCheckable(true);
+    viewFullscreenAction->setChecked(false);
+    viewFullscreenAction->setShortcut(Qt::Key_F11);
+
+    viewMenu->addAction(viewFullscreenAction);
+
+
     this->menuBar()->addMenu(viewMenu);
 
 
@@ -614,6 +709,16 @@ void MainWindow::createMenus()
             this, SLOT(visitWebSite()));
     helpMenu->addAction(helpVisitWebsite);
 
+
+    helpVisitPumpFAQ = new QAction(QIcon::fromTheme("internet-web-browser"),
+                                  tr("&Frequently Asked Questions about Pump.io"), this);
+    connect(helpVisitPumpFAQ, SIGNAL(triggered()),
+            this, SLOT(visitFAQ()));
+    helpMenu->addAction(helpVisitPumpFAQ);
+
+
+    helpMenu->addSeparator();
+
     helpAbout = new QAction(QIcon::fromTheme("system-help"),
                             tr("About &Dianara"), this);
     connect(helpAbout, SIGNAL(triggered()),
@@ -630,7 +735,9 @@ void MainWindow::createMenus()
                                this, SLOT(show()));
     trayContextMenu->addSeparator();
     trayContextMenu->addAction(sessionUpdateMainTimeline);
+    trayContextMenu->addAction(sessionMarkAllAsRead);
     trayContextMenu->addAction(sessionPostNote);
+    trayContextMenu->addSeparator();
     trayContextMenu->addAction(settingsConfigure);
     trayContextMenu->addAction(helpAbout);
     trayContextMenu->addSeparator();
@@ -665,6 +772,9 @@ void MainWindow::createTrayIcon()
         connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
                 this, SLOT(trayControl(QSystemTrayIcon::ActivationReason)));
 
+        // clicking in a popup notification (balloon-type) will show the window
+        connect(trayIcon, SIGNAL(messageClicked()),
+                this, SLOT(show()));
 
         // Set contextual menu for the icon
         trayIcon->setContextMenu(this->trayContextMenu);
@@ -712,6 +822,52 @@ void MainWindow::setTrayIconPixmap(int count)
 }
 
 
+/*
+ * Adjust timelines maximum widths according to their scrollareas sizes,
+ * which in turn depend on the window size
+ *
+ * Called from the resizeEvent(), among other places
+ *
+ */
+void MainWindow::adjustTimelineSizes()
+{
+    int scrollbarWidth = mainTimelineScrollArea->verticalScrollBar()->width() + 4;
+    // FIXME: adding +4 pixels should not be hardcoded
+
+    int timelineWidth;
+
+    // Get the right timelinewidth based on currectly active tab's timeline width
+    switch (this->tabWidget->currentIndex())
+    {
+    case 0: // main
+        timelineWidth = mainTimelineScrollArea->width() - scrollbarWidth;
+        mainTimeline->resizePosts();
+        break;
+    case 1: // direct
+        timelineWidth = directTimelineScrollArea->width() - scrollbarWidth;
+        directTimeline->resizePosts();
+        break;
+    case 2: // activity
+        timelineWidth = activityTimelineScrollArea->width() - scrollbarWidth;
+        activityTimeline->resizePosts();
+        break;
+    case 3: // favorites
+        timelineWidth = favoritesTimelineScrollArea->width() - scrollbarWidth;
+        favoritesTimeline->resizePosts();
+        break;
+
+    default: // Contacts tab, ATM
+        timelineWidth = contactList->width() - scrollbarWidth;
+    }
+
+    // Then set the maximum width for all of them based on that
+    mainTimeline->setMaximumWidth(timelineWidth);
+    directTimeline->setMaximumWidth(timelineWidth);
+    activityTimeline->setMaximumWidth(timelineWidth);
+    favoritesTimeline->setMaximumWidth(timelineWidth);
+}
+
+
 
 /*
  * Load general program settings and state: size, position...
@@ -849,10 +1005,11 @@ void MainWindow::updateUserID(QString newUserID)
     this->fullNameLabel->setText("--");
     this->userIdLabel->setText("_@_");
     this->userHometownLabel->setText("--");
-    avatarIconLabel->setPixmap(QPixmap(":/images/no-avatar.png")
-                             .scaled(64, 64,
-                                     Qt::KeepAspectRatio,
-                                     Qt::SmoothTransformation));
+
+    avatarIconButton->setIcon(QIcon(QPixmap(":/images/no-avatar.png")
+                                    .scaled(64, 64,
+                                            Qt::KeepAspectRatio,
+                                            Qt::SmoothTransformation)));
 
 
     qDebug() << "UserID updated from AccountDialog:" << userID;
@@ -922,7 +1079,7 @@ void MainWindow::trayControl(QSystemTrayIcon::ActivationReason reason)
         // Hide or show the main window
         if (this->isMinimized())
         {
-            // hide and show, because raise() wouln't work
+            // hide and show, because raise() wouldn't work
             this->hide();
             this->showNormal();
             qDebug() << "RAISING!";
@@ -983,17 +1140,18 @@ void MainWindow::updateProfileData(QString avatarURL, QString fullName,
     if (QFile::exists(avatarFilename))
     {
         // Load avatar if already cached
-        this->avatarIconLabel->setPixmap(QPixmap(avatarFilename)
-                                                .scaled(64, 64,
-                                                        Qt::KeepAspectRatio,
-                                                        Qt::SmoothTransformation));
+        this->avatarIconButton->setIcon(QIcon(QPixmap(avatarFilename)
+                                              .scaled(64, 64,
+                                                      Qt::KeepAspectRatio,
+                                                      Qt::SmoothTransformation)));
+
         qDebug() << "Using cached avatar for user";
     }
     else
     {
         pumpController->getAvatar(avatarURL);
     }
-    this->avatarIconLabel->setToolTip(bio);
+    this->avatarIconButton->setToolTip(bio);
 
 
     // Fill/update this info in the profile editor too
@@ -1051,24 +1209,57 @@ void MainWindow::scrollMainTimelineToTop()
 {
     this->mainTimelineScrollArea->verticalScrollBar()->setValue(0);
     this->mainTimelineScrollArea->horizontalScrollBar()->setValue(0);
+
+    this->adjustTimelineSizes(); // kinda TMP
 }
 
 void MainWindow::scrollDirectTimelineToTop()
 {
     this->directTimelineScrollArea->verticalScrollBar()->setValue(0);
     this->directTimelineScrollArea->horizontalScrollBar()->setValue(0);
+
+    this->adjustTimelineSizes();
 }
 
 void MainWindow::scrollActivityTimelineToTop()
 {
     this->activityTimelineScrollArea->verticalScrollBar()->setValue(0);
     this->activityTimelineScrollArea->horizontalScrollBar()->setValue(0);
+
+    this->adjustTimelineSizes();
 }
 
 void MainWindow::scrollFavoritesTimelineToTop()
 {
     this->favoritesTimelineScrollArea->verticalScrollBar()->setValue(0);
     this->favoritesTimelineScrollArea->horizontalScrollBar()->setValue(0);
+
+    this->adjustTimelineSizes();
+}
+
+/*
+ * Scroll timelines to make sure the commenter block of the post
+ * currently being commented is shown
+ *
+ */
+void MainWindow::scrollMainTimelineToWidget(QWidget *widget)
+{
+    this->mainTimelineScrollArea->ensureWidgetVisible(widget, 1, 1);
+}
+
+void MainWindow::scrollDirectTimelineToWidget(QWidget *widget)
+{
+    this->directTimelineScrollArea->ensureWidgetVisible(widget, 1, 1);
+}
+
+void MainWindow::scrollActivityTimelineToWidget(QWidget *widget)
+{
+    this->activityTimelineScrollArea->ensureWidgetVisible(widget, 1, 1);
+}
+
+void MainWindow::scrollFavoritesTimelineToWidget(QWidget *widget)
+{
+    this->favoritesTimelineScrollArea->ensureWidgetVisible(widget, 1, 1);
 }
 
 
@@ -1094,7 +1285,7 @@ void MainWindow::notifyTimelineUpdate(int timelineType, int newPostCount)
            && timelineType == TimeLine::TimelineTypeMain)
         {
             this->fdNotifier->showMessage(tr("Timeline updated at %1.").arg(QTime::currentTime().toString())
-                                        + " " + newPostsString);
+                                        + "\n" + newPostsString);
         }
     }
     else
@@ -1153,13 +1344,20 @@ void MainWindow::setTimelineTabTitle(int timelineType, int newPostCount)
 
     // If the updated tab is the current one, set also window title, etc.
     if (updatedTab == tabWidget->currentIndex())
+    {
+        this->setTitleAndTrayInfo(updatedTab);
+    }
+
+
+    // If it's the main timeline, set the tray icon pixmap with the newmsg count
+    if (updatedTab == 0)
     {
         if (trayIconAvailable)
         {
             this->setTrayIconPixmap(newPostCount);
-            this->setTitleAndTrayInfo(updatedTab);
         }
     }
+
 }
 
 
@@ -1224,13 +1422,16 @@ void MainWindow::storeAvatar(QByteArray avatarData, QUrl avatarURL)
     avatarFile.write(avatarData);
     avatarFile.close();
 
+    this->pumpController->notifyAvatarStored(avatarURL.toString(),
+                                             avatarFile.fileName());
+
     if (avatarURL == this->avatarURL)
     {
-        this->avatarIconLabel->setPixmap(QPixmap(dataDirectory + "/avatars/" + fileName)
-                                         .scaled(64, 64,
-                                                 Qt::KeepAspectRatio,
-                                                 Qt::SmoothTransformation));
-    }        
+        this->avatarIconButton->setIcon(QIcon(QPixmap(avatarFile.fileName())
+                                              .scaled(64, 64,
+                                                      Qt::KeepAspectRatio,
+                                                      Qt::SmoothTransformation)));
+    }
 
     qDebug() << "avatarData size:" << avatarData.size();
 }
@@ -1255,6 +1456,8 @@ void MainWindow::storeImage(QByteArray imageData, QUrl imageURL)
     imageFile.write(imageData);
     imageFile.close();
 
+    this->pumpController->notifyImageStored(imageURL.toString());
+
     qDebug() << "imageData size:" << imageData.size();
 }
 
@@ -1287,6 +1490,12 @@ void MainWindow::toggleSidePanel(bool shown)
 
     qDebug() << "Showing side panel:" << shown;
     this->leftSideWidget->setVisible(shown);
+
+    /*if (!shown)
+    {
+        this->adjustTimelineSizes();
+    }
+    */
 }
 
 
@@ -1301,6 +1510,23 @@ void MainWindow::toggleStatusBar(bool shown)
 }
 
 
+/*
+ * Toggle between fullscreen mode and normal window mode
+ *
+ */
+void MainWindow::toggleFullscreen(bool enabled)
+{
+    if (enabled)
+    {
+        this->showFullScreen();
+    }
+    else
+    {
+        this->showNormal();
+    }
+}
+
+
 
 
 
@@ -1314,6 +1540,17 @@ void MainWindow::visitWebSite()
     QDesktopServices::openUrl(QUrl("http://jancoding.wordpress.com/dianara"));
 }
 
+/*
+ * Open pump.io's FAQ in web browser
+ * (currently at GitHub)
+ *
+ */
+void MainWindow::visitFAQ()
+{
+    qDebug() << "Opening Pump.io FAQ in browser";
+    QDesktopServices::openUrl(QUrl("https://github.com/e14n/pump.io/wiki/FAQ"));
+}
+
 
 
 
@@ -1324,7 +1561,7 @@ void MainWindow::visitWebSite()
 void MainWindow::aboutDianara()
 {
     QMessageBox::about(this, tr("About Dianara"),
-                       "<b>Dianara v0.9</b><br>"
+                       "<b>Dianara v1.0</b><br>"
                        "Copyright 2012-2013  JanKusanagi<br><br>"
                        "<a href=\"http://jancoding.wordpress.com/dianara\">"
                        "http://jancoding.wordpress.com/dianara</a><br><br>"
@@ -1335,10 +1572,20 @@ void MainWindow::aboutDianara()
                        + tr("With Dianara you can see your timelines, create new posts, "
                             "upload pictures, interact with posts, manage "
                             "your contacts and follow new people.")
-                       + "<br><br><br>"
+                       + "<br><br>"
+
+                       + tr("Thanks to all the testers, translators and packagers, "
+                            "who help make Dianara better!")
+                       + "<br><br>"
 
                        + tr("English translation by JanKusanagi.",
-                            "TRANSLATORS: Change this with your language and name ;)"));
+                            "TRANSLATORS: Change this with your language and name ;)")
+                       + "<br><br>"
+
+                       + tr("Dianara is licensed under the GNU GPL license, and "
+                            "uses some Oxygen icons: http://www.oxygen-icons.org/ "
+                            "(LGPL licensed)"));
+
 
     if (this->isHidden()) // FIXME: ugly workaround to avoid closing the program
     {                     // after closing About dialog, if mainWindow hidden
