@@ -78,6 +78,8 @@ public:
     void createTrayIcon();
     void setTrayIconPixmap(int count = 0);
 
+    void adjustTimelineSizes();
+
     void loadSettings();
     void saveSettings();
 
@@ -106,6 +108,12 @@ public slots:
     void scrollDirectTimelineToTop();
     void scrollActivityTimelineToTop();
     void scrollFavoritesTimelineToTop();
+
+    void scrollMainTimelineToWidget(QWidget *widget);
+    void scrollDirectTimelineToWidget(QWidget *widget);
+    void scrollActivityTimelineToWidget(QWidget *widget);
+    void scrollFavoritesTimelineToWidget(QWidget *widget);
+
     void notifyTimelineUpdate(int timelineType, int newPostCount);
     void setTimelineTabTitle(int timelineType, int newPostCount);
 
@@ -119,15 +127,19 @@ public slots:
 
     void toggleSidePanel(bool shown);
     void toggleStatusBar(bool shown);
+    void toggleFullscreen(bool enabled);
 
 
     void visitWebSite();
+    void visitFAQ();
     void aboutDianara();
+
     void quitProgram();
 
 
 protected:
     virtual void closeEvent(QCloseEvent *event);
+    virtual void resizeEvent(QResizeEvent *event);
 
 
 private:
@@ -145,17 +157,20 @@ private:
     QAction *sessionUpdateFavoritesTimeline;
     QAction *sessionUpdateMinorFeed;
     QAction *sessionUpdateAllTimelines;
+    QAction *sessionMarkAllAsRead;
     QAction *sessionPostNote;
     QAction *sessionQuit;
 
     QAction *viewSidePanel;
     QAction *viewStatusBar;
+    QAction *viewFullscreenAction;
 
     QAction *settingsEditProfile;
     QAction *settingsAccount;
     QAction *settingsConfigure;
 
     QAction *helpVisitWebsite;
+    QAction *helpVisitPumpFAQ;
     QAction *helpAbout;
 
 
@@ -171,7 +186,7 @@ private:
     int tabsPosition;
     bool tabsMovable;
 
-    QLabel *avatarIconLabel;
+    QPushButton *avatarIconButton;
     QString avatarURL;
     QLabel *fullNameLabel;
     QLabel *userIdLabel;
