@@ -27,10 +27,21 @@ MinorFeed::MinorFeed(PumpController *pumpController,
     this->pController = pumpController;
     this->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
 
-
     mainLayout = new QVBoxLayout();
+    mainLayout->setAlignment(Qt::AlignTop);
     this->setLayout(mainLayout);
 
+    MinorFeedItem *demoFeedItem = new MinorFeedItem(QDateTime::currentDateTimeUtc()
+                                                              .toString(Qt::ISODate),
+                                                    "Dianara",
+                                                    tr("There are no activities to show yet."),
+                                                    "",
+                                                    "",
+                                                    "");
+    this->mainLayout->addWidget(demoFeedItem);
+    this->itemsInFeed.append(demoFeedItem);
+
+
     qDebug() << "MinorFeed created";
 }
 
@@ -71,6 +82,7 @@ void MinorFeed::setFeedContents(QVariantList activitiesList)
     this->clearContents();
 
     QString activityTime;
+    QString activityGenerator;
     QString activityDescription;
     QString activityObjectContent;
     QString activityObjectAuthor;
@@ -82,21 +94,23 @@ void MinorFeed::setFeedContents(QVariantList activitiesList)
     {
         QVariantMap activityMap = activityVariant.toMap();
 
-        activityTime = activityMap["updated"].toString();
-        activityDescription = activityMap["content"].toString();
+        activityTime = activityMap.value("updated").toString();
+        activityGenerator = activityMap.value("generator").toMap().value("displayName").toString();
+        activityDescription = activityMap.value("content").toString();
 
-        activityActorAvatarURL = activityMap["actor"].toMap()["image"].toMap()["url"].toString();
+        activityActorAvatarURL = activityMap.value("actor").toMap().value("image").toMap().value("url").toString();
         activityActorAvatarFilename = MiscHelpers::getCachedAvatarFilename(activityActorAvatarURL);
         if (!QFile::exists(activityActorAvatarFilename))
         {
             this->pController->getAvatar(activityActorAvatarURL);
         }
 
-        activityObjectContent = activityMap["object"].toMap()["content"].toString();
-        activityObjectAuthor = activityMap["object"].toMap()["author"].toMap()["displayName"].toString();
+        activityObjectContent = activityMap.value("object").toMap().value("content").toString();
+        activityObjectAuthor = activityMap.value("object").toMap().value("author").toMap().value("displayName").toString();
 
 
         MinorFeedItem *newFeedItem = new MinorFeedItem(activityTime,
+                                                       activityGenerator,
                                                        activityDescription,
                                                        activityActorAvatarFilename,
                                                        activityObjectAuthor,
