@@ -68,6 +68,67 @@ QString MiscHelpers::getCachedImageFilename(QString url)
 }
 
 
+/*
+ * Return MIME content type, like image/png, etc.
+ *
+ *
+ */
+QString MiscHelpers::getImageContentType(QString fileURI)
+{
+    // QImageReader::setDecideFormatFromContent(true) ?
+    QString formatString = QString::fromLocal8Bit(QImageReader::imageFormat(fileURI));
+    qDebug() << "Image format:" << formatString;
+
+    if (formatString == "png" || formatString == "jpeg" || formatString == "gif")
+    {
+        return QString("image/" + formatString);
+    }
+    else
+    {
+        return QString();
+    }
+}
+
+/*
+ * Return width of an image
+ *
+ */
+int MiscHelpers::getImageWidth(QString fileURI)
+{
+    QImageReader imageReader(fileURI);
+
+    return imageReader.size().width();
+}
+
+
+
+/*
+ * Return a pretty string with the size of a file, like
+ * "33 KiB", "512 bytes" or "3 MiB"
+ *
+ */
+QString MiscHelpers::fileSizeString(QString fileURI)
+{
+    QFileInfo fileInfo(fileURI);
+    int fileSize = fileInfo.size();
+
+    QString sizeUnit = tr("bytes");
+    if (fileSize > 1024) // if > 1024 bytes, transform to KiB
+    {
+        fileSize /= 1024;
+        sizeUnit = "KiB";
+    }
+    if (fileSize > 1024) // if > 1024 KiB, transform to MiB
+    {
+        fileSize /= 1024;
+        sizeUnit = "MiB";
+    }
+
+
+    return QString("%1 %2").arg(fileSize).arg(sizeUnit);
+}
+
+
 
 
 /*
@@ -77,7 +138,7 @@ QString MiscHelpers::getCachedImageFilename(QString url)
  * Return also the string list of the URL's to download
  *
  */
-QStringList MiscHelpers::htmlWithReplacedImages(QString originalHtml)
+QStringList MiscHelpers::htmlWithReplacedImages(QString originalHtml, int postWidth)
 {
     // if no <img tag is found, just return the original HTML
     if (!originalHtml.contains("<img"))
@@ -85,13 +146,13 @@ QStringList MiscHelpers::htmlWithReplacedImages(QString originalHtml)
         return QStringList(originalHtml);
     }
 
-    qDebug() << "MiscHelpers::htmlWithReplacedImages(); HTML contains some <img> tags...";
+    //qDebug() << "MiscHelpers::htmlWithReplacedImages(); HTML contains some <img> tags...";
     QString newHtml = originalHtml;
 
     QStringList imageList;
     QString imgSrc;
 
-    QRegExp regExp("\\<img(.+)src=\"([^\"]+)\"\\s(.*)\\>");
+    QRegExp regExp("\\<img(.+)src=\"([^\"]+)\"(.*)\\>");
     regExp.setMinimal(true);
 
     int matchedLength;
@@ -102,25 +163,35 @@ QStringList MiscHelpers::htmlWithReplacedImages(QString originalHtml)
         matchedLength = regExp.matchedLength();
 
 
-        qDebug() << "regExp match = " << regExp.cap(0);
-        qDebug() << "Groups:" << regExp.cap(1) << " // " << regExp.cap(2) << " // " << regExp.cap(3);
-        qDebug() << "Matched length is:" << matchedLength;
+        //qDebug() << "regExp match = " << regExp.cap(0);
+        //qDebug() << "Groups:" << regExp.cap(1) << " // " << regExp.cap(2) << " // " << regExp.cap(3);
+        //qDebug() << "Matched length is:" << matchedLength;
 
         imgSrc = regExp.cap(2);
         if (!imgSrc.isEmpty()) // if not an empty string, add to the list, and replace HTML
         {
             imageList.append(imgSrc);
 
+            QString cachedImageFilename = getCachedImageFilename(imgSrc);
+
+            int imageWidth = getImageWidth(cachedImageFilename);
+            // if width is bigger than the post, make it smaller to fit
+            if (imageWidth > postWidth - 40)
+            {
+                imageWidth = postWidth - 40; // Some margins, to account for a scrollbar
+            }
+
             newHtml.replace(stringPos,
                             matchedLength,
-                            "<img src=\"" + getCachedImageFilename(imgSrc) +  "\" width=\"320\"/>");
+                            "<img src=\"" + cachedImageFilename +  "\" "
+                            "width=\"" + QString("%1").arg(imageWidth) + "\" />");
         }
 
         stringPos += matchedLength; // FIXME: error control
     }
 
     imageList.insert(0, newHtml); // The modified HTML goes before the image list
-    qDebug() << "Returned HTML and images:\n" << imageList;
+    //qDebug() << "Returned HTML and images:\n" << imageList;
 
     return imageList;
 }
@@ -130,7 +201,7 @@ QStringList MiscHelpers::htmlWithReplacedImages(QString originalHtml)
  * Basic cleanup of HTML stuff
  *
  */
-QString MiscHelpers::cleanHtml(QString originalHtml)
+QString MiscHelpers::cleanupHtml(QString originalHtml)
 {
     QString cleanHtml = originalHtml;
 
@@ -151,3 +222,26 @@ QString MiscHelpers::cleanHtml(QString originalHtml)
 
     return cleanHtml;
 }
+
+
+
+/*
+ * Return some HTML with a blockquote, quote symbols, etc.
+ *
+ *
+ */
+QString MiscHelpers::quotedText(QString author, QString content)
+{
+    QTextDocument textDocument;
+    textDocument.setHtml(content);
+
+    content = textDocument.toPlainText().trimmed();
+    content.replace("\n", "<br>");
+
+    QString quoteHtml = "&gt;&gt; <b><u>"+ author + ":</u></b>"  // >>> + name
+                        "<blockquote>"
+                        "<small><i>&ldquo;" + content + "&rdquo;</i></small>"
+                        "</blockquote> <br>";
+
+    return quoteHtml;
+}
