@@ -30,7 +30,7 @@ Post::Post(PumpController *pumpController,
     this->pController = pumpController;
 
     this->setSizePolicy(QSizePolicy::MinimumExpanding,
-                        QSizePolicy::Maximum);
+                        QSizePolicy::Minimum);
 
 
     leftColumnLayout = new QVBoxLayout();
@@ -38,6 +38,18 @@ Post::Post(PumpController *pumpController,
 
     this->postID = postData.value("id");
     this->postType = postData.value("postType");
+    this->postURL = postData.value("url");
+    this->postAuthorID = postData.value("authorId");
+    this->postAuthorName = postData.value("author");
+    if (postAuthorName.isEmpty()) // In case name is empty, use webfinger ID
+    {
+        postAuthorName = this->postAuthorID;
+    }
+    this->postAuthorURL = postData.value("authorUrl");
+
+    // Save post title for openClickedURL(), etc
+    this->postTitle = postData.value("title");
+
 
     postIsUnread = false;
     if (postData.value("postIsNew") == "true")
@@ -76,7 +88,7 @@ Post::Post(PumpController *pumpController,
     leftColumnLayout->addSpacing(2);
 
 
-    QString authorTooltipInfo = "<b>" + postData.value("authorId") + "</b>";
+    QString authorTooltipInfo = "<b>" + postAuthorID + "</b>";
     authorTooltipInfo.append("<br>");
     if (!postData.value("authorHometown").isEmpty())
     {
@@ -91,7 +103,7 @@ Post::Post(PumpController *pumpController,
     }
     authorTooltipInfo.replace("\n", "<br>"); // HTML newlines
 
-    if (postData.value("authorId") == pController->currentUserID())
+    if (this->postAuthorID == pController->currentUserID())
     {
         postIsOwn = true;
         qDebug() << "post is our own!";
@@ -105,33 +117,43 @@ Post::Post(PumpController *pumpController,
     }
 
     // Author avatar
-    postAuthorAvatar = new QLabel();
+    postAuthorAvatarButton = new QPushButton();
+    postAuthorAvatarButton->setSizePolicy(QSizePolicy::Maximum,
+                                          QSizePolicy::Maximum);
+    postAuthorAvatarButton->setFlat(true);
+    postAuthorAvatarButton->setIconSize(QSize(48,48));
+    postAuthorAvatarButton->setToolTip(authorTooltipInfo);
 
-    // Get local file name, which is stored in base64 hash form
-    QString avatarFilename = MiscHelpers::getCachedAvatarFilename(postData.value("authorAvatar"));
+    // Get "following" status before creating avatar menu
+    this->postAuthorFollowed = this->pController->userInFollowing(this->postAuthorID);
+    this->createAvatarMenu();
+    postAuthorAvatarButton->setMenu(avatarMenu);
 
+    this->postAuthorAvatarUrl = postData.value("authorAvatar");
 
-    if (QFile::exists(avatarFilename) && !postData.value("authorAvatar").isEmpty())
+    // Get local file name for avatar, which is stored in base64 hash form
+    QString avatarFilename = MiscHelpers::getCachedAvatarFilename(postAuthorAvatarUrl);
+
+    if (QFile::exists(avatarFilename) && !postAuthorAvatarUrl.isEmpty())
     {
         // Load avatar if already cached
-        postAuthorAvatar->setPixmap(QPixmap(avatarFilename)
-                                    .scaled(48, 48,
-                                            Qt::KeepAspectRatio,
-                                            Qt::SmoothTransformation));
-        qDebug() << "Using cached avatar";
+        postAuthorAvatarButton->setIcon(QIcon(QPixmap(avatarFilename).scaled(48, 48,
+                                                                  Qt::KeepAspectRatio,
+                                                                  Qt::SmoothTransformation)));
     }
     else
     {
         // Placeholder image
-        postAuthorAvatar->setPixmap(QIcon::fromTheme("user-identity").pixmap(48,48));
+        postAuthorAvatarButton->setIcon(QIcon::fromTheme("user-identity",
+                                                         QIcon(":/images/no-avatar.png")));
         qDebug() << "Using placeholder, downloading real avatar now";
 
         // Download avatar for next time
-        pumpController->getAvatar(postData.value("authorAvatar"));
+        connect(pController, SIGNAL(avatarStored(QString,QString)),
+                this, SLOT(redrawAvatar(QString,QString)));
+        pController->getAvatar(postAuthorAvatarUrl);
     }
-    postAuthorAvatar->setToolTip(authorTooltipInfo);
-
-    leftColumnLayout->addWidget(postAuthorAvatar);
+    leftColumnLayout->addWidget(postAuthorAvatarButton, 0, Qt::AlignLeft);
     leftColumnLayout->addSpacing(2);
 
 
@@ -139,14 +161,13 @@ Post::Post(PumpController *pumpController,
     authorFont.setBold(true);
     authorFont.setUnderline(true);
     authorFont.setPointSize(authorFont.pointSize()-1);
-
-    // Author name
-    QString authorName = postData.value("author");
-    if (authorName.isEmpty()) // In case name is empty, use webfinger ID
+    if (postIsOwn) // Another visual hint when the post is our own
     {
-        authorName = postData.value("authorId");
+        authorFont.setItalic(true);
     }
 
+    // Author name
+    QString authorName = this->postAuthorName;
     if (authorName.length() > 16)   // very TMP optimization of LOOONG names / FIXME
     {
         authorName.replace("@", "@ ");
@@ -163,13 +184,24 @@ Post::Post(PumpController *pumpController,
     leftColumnLayout->addSpacing(2);
 
 
-
     // Post creation time
-    QString createdAt = postData.value("createdAt");
+    QString createdAt = postData["createdAt"];
+    // and update time
+    QString updatedAt = postData["updatedAt"];
+    if (updatedAt == createdAt)
+    {
+        updatedAt.clear();
+    }
     // Format is "Combined date and time in UTC", like "2012-02-07T01:32:02Z"
     // as specified in ISO 8601 http://en.wikipedia.org/wiki/ISO_8601
     QString exactPostTime = Timestamp::localTimeDate(createdAt);
     createdAt = Timestamp::fuzzyTime(createdAt);
+    if (!updatedAt.isEmpty())
+    {
+        createdAt.append("\n");
+        createdAt.append(tr("Edited: %1").arg(Timestamp::fuzzyTime(updatedAt)));
+    }
+
     QString generator = postData.value("generator");
 
     postCreatedAtLabel = new QLabel(createdAt);
@@ -195,24 +227,24 @@ Post::Post(PumpController *pumpController,
     postToLabel = new QLabel(tr("To") + ": " + postData.value("to"));
     postToLabel->setWordWrap(true);
     postToLabel->setFont(detailsFont);
-    leftColumnLayout->addWidget(postToLabel);
+    postToLabel->setOpenExternalLinks(true);
+    if (!postData["to"].isEmpty())
+    {
+        leftColumnLayout->addWidget(postToLabel);
+    }
 
     postCCLabel = new QLabel(tr("CC") + ": " + postData.value("cc"));
     postCCLabel->setWordWrap(true);
     postCCLabel->setFont(detailsFont);
-    leftColumnLayout->addWidget(postCCLabel);
+    postCCLabel->setOpenExternalLinks(true);
+    if (!postData["cc"].isEmpty())
+    {
+        leftColumnLayout->addWidget(postCCLabel);
+    }
+
     leftColumnLayout->addSpacing(4);
 
 
-    openInBrowserLabel = new QLabel("<a href=\"" + postData.value("url") + "\">" + tr("Open") +" &#9658;</a>");
-    openInBrowserLabel->setToolTip(tr("Open this post in your web browser"));
-    openInBrowserLabel->setOpenExternalLinks(true);
-    openInBrowserLabel->setFont(detailsFont);
-    leftColumnLayout->addWidget(openInBrowserLabel);
-
-    leftColumnLayout->addSpacing(8);
-
-
     this->postLikesURL = postData.value("likesURL");
     this->postCommentsURL = postData.value("commentsURL");
 
@@ -246,8 +278,8 @@ Post::Post(PumpController *pumpController,
     postResharesCountLabel->setFont(detailsFont);
     leftColumnLayout->addWidget(postResharesCountLabel);
     postResharesCountLabel->hide();
-    int resharesCount = postData.value("resharesCount").toInt();
-    this->setSharesLabel(resharesCount);
+    int sharesCount = postData.value("resharesCount").toInt();
+    this->setSharesLabel(sharesCount);
 
 
     // Try to use all remaining space, aligning
@@ -256,6 +288,7 @@ Post::Post(PumpController *pumpController,
     leftColumnLayout->addStretch(1);
 
 
+
     //////////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////// Right column, content
@@ -263,11 +296,19 @@ Post::Post(PumpController *pumpController,
     rightColumnLayout = new QVBoxLayout();
     rightColumnLayout->setAlignment(Qt::AlignRight | Qt::AlignTop);
 
+    ///////////////////////////////////// Title
+    this->postTitleLabel = new QLabel();
+    postTitleLabel->setWordWrap(true);
+    QFont postTitleFont;
+    postTitleFont.setBold(true);
+    postTitleFont.setPointSize(postTitleFont.pointSize() + 1);
+    postTitleLabel->setFont(postTitleFont);
+    postTitleLabel->setText(postTitle);
+
 
     ///////////////////////////////////// Post text
     postText = new QTextBrowser();
     postText->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-    //postText->setFrameStyle(QFrame::StyledPanel | QFrame::Raised); // Raised style? :-m
     postText->setSizePolicy(QSizePolicy::MinimumExpanding,
                             QSizePolicy::MinimumExpanding);
 
@@ -280,8 +321,6 @@ Post::Post(PumpController *pumpController,
             this, SLOT(showHighlightedURL(QUrl)));
 
     // Add a label in the bottom right corner, to show where links go
-    postTextLayout = new QHBoxLayout();
-
     highlightedUrlLabel = new QLabel();
     highlightedUrlLabel->setAlignment(Qt::AlignLeft);
     highlightedUrlLabel->setFont(detailsFont);
@@ -290,23 +329,30 @@ Post::Post(PumpController *pumpController,
     highlightedUrlLabel->setAutoFillBackground(true);
     highlightedUrlLabel->setWordWrap(true);
     highlightedUrlLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
-
-    postTextLayout->addWidget(highlightedUrlLabel, 0, Qt::AlignRight | Qt::AlignBottom);
     highlightedUrlLabel->hide(); // Hidden initially
+
+    postTextLayout = new QVBoxLayout();
+    postTextLayout->setAlignment(Qt::AlignRight | Qt::AlignBottom);
+    postTextLayout->addWidget(highlightedUrlLabel);
     postText->setLayout(postTextLayout);
 
+    QFont buttonsFont;
+    buttonsFont.setPointSize(buttonsFont.pointSize() - 1);
 
     // Like, comment, share buttons
-    likeButton = new QPushButton(QIcon::fromTheme("emblem-favorite"), "");
+    likeButton = new QPushButton(QIcon::fromTheme("emblem-favorite"), "like");
     likeButton->setCheckable(true);
     this->fixLikeButton(postData.value("postIsLiked"));
     likeButton->setFlat(true);
+    likeButton->setFont(buttonsFont);
     connect(likeButton, SIGNAL(clicked(bool)),
             this, SLOT(likePost(bool)));
 
     commentButton = new QPushButton(QIcon::fromTheme("mail-reply-sender"), tr("Comment"));
-    commentButton->setToolTip(tr("Comment on this post"));
+    commentButton->setToolTip(tr("Comment on this post.") + "<br>"
+                              + tr("If you select some text, it will be quoted."));
     commentButton->setFlat(true);
+    commentButton->setFont(buttonsFont);
     connect(commentButton, SIGNAL(clicked()),
             this, SLOT(commentOnPost()));
 
@@ -315,12 +361,13 @@ Post::Post(PumpController *pumpController,
                                   tr("Share"));
     shareButton->setToolTip(tr("Share this post"));
     shareButton->setFlat(true);
+    shareButton->setFont(buttonsFont);
     connect(shareButton, SIGNAL(clicked()),
             this, SLOT(sharePost()));
 
 
 
-    // Add like, comment, share and, if post is our own, delete buttons
+    // Add like, comment, share and, if post is our own, edit and delete buttons
     buttonsLayout = new QHBoxLayout();
     buttonsLayout->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
     buttonsLayout->setContentsMargins(0, 0, 0, 0);
@@ -332,13 +379,22 @@ Post::Post(PumpController *pumpController,
         buttonsLayout->addWidget(commentButton, 0, Qt::AlignLeft);
         buttonsLayout->addWidget(shareButton,   0, Qt::AlignLeft);
     }
-    buttonsLayout->addStretch(1); // so the (optional) Delete button gets separated
+    buttonsLayout->addStretch(1); // so the (optional) Edit and Delete buttons get separated
 
     if (postIsOwn)
     {
+        editButton = new QPushButton(QIcon::fromTheme("document-edit"), tr("Edit"));
+        editButton->setToolTip(tr("Edit this post"));
+        editButton->setFlat(true);
+        editButton->setFont(buttonsFont);
+        connect(editButton, SIGNAL(clicked()),
+                this, SLOT(editPost()));
+        buttonsLayout->addWidget(editButton, 0, Qt::AlignRight);
+
         deleteButton = new QPushButton(QIcon::fromTheme("edit-delete"), tr("Delete"));
         deleteButton->setToolTip(tr("Delete this post"));
         deleteButton->setFlat(true);
+        deleteButton->setFont(buttonsFont);
         connect(deleteButton, SIGNAL(clicked()),
                 this, SLOT(deletePost()));
 
@@ -346,47 +402,21 @@ Post::Post(PumpController *pumpController,
     }
 
 
+    /******************************************************************/
+
 
     // Get URL of post image, if it's "image" type of post
-    QString postTitle;
-    QString postImage;
     if (this->postType == "image")
     {
-        postImage = postData.value("image");
+        postImageURL = postData.value("image");
 
-        if (QFile::exists(MiscHelpers::getCachedImageFilename(postImage)))
-        {
-            qDebug() << "Using cached image";
-        }
-        else
-        {
-            qDebug() << "Image not cached, downloading" << postImage;
-            pumpController->getImage(postImage);
-        }
-
-        this->postImageTitle = postData.value("title");
-        // If title's not empty, add the header tags and line break,
-        // otherwhise don't add useless <br>
-        if (!postImageTitle.isEmpty())
-        {
-            postTitle = "<h3>" + postImageTitle + "</h3><br>";
-        }
-        postImage = "<a href=\"image:/" + MiscHelpers::getCachedImageFilename(postImage) + "\">"
-                    "<img src=\"" + MiscHelpers::getCachedImageFilename(postImage) + "\" width=\"320\" />"
-                    "</a>"
-                    "<br><br>";
+        this->enqueueImageForDownload(postImageURL);
     }
 
-
-    QStringList postTextImageList = MiscHelpers::htmlWithReplacedImages(postData.value("text"));
-    QString postTextContents = postTextImageList.takeAt(0);
-
-
-    // Add the text content of the post
-    postText->setHtml(postTitle
-                    + postImage
-                    + postTextContents);
-
+    this->postOriginalText = postData.value("text"); // Save it for later...
+    QStringList postTextImageList = MiscHelpers::htmlWithReplacedImages(postOriginalText,
+                                                                        this->postText->width());
+    postTextImageList.removeFirst(); // First is the HTML itself
 
     // If the image list is not empty, get them (unless they're cached already)
     if (!postTextImageList.isEmpty())
@@ -395,34 +425,14 @@ Post::Post(PumpController *pumpController,
 
         foreach (QString imageUrl, postTextImageList)
         {
-            if (QFile::exists(MiscHelpers::getCachedImageFilename(imageUrl)))
-            {
-                qDebug() << "Image already cached, ignoring";
-            }
-            else
-            {
-                qDebug() << "Image not cached, downloading";
-                this->pController->getImage(imageUrl);
-            }
+            this->enqueueImageForDownload(imageUrl);
         }
     }
 
 
-    int postLines = postText->document()->lineCount() + 1;
-    qDebug() << "Post lines:" << postLines;
+    // Post resizing takes place in resizeEvent()
+    // which will also call setPostContents()
 
-    int postMinimumHeight = postLines * 2 * postText->fontInfo().pixelSize();
-    if (postType == "image")
-    {
-        postMinimumHeight += 100;
-    }
-    if (postMinimumHeight > 100)  // Don't allow a post to have a minimum height > 200 px
-    {
-        postMinimumHeight = 100;  // Scrollbars will appear
-        qDebug() << "Post would require more space: scrollbars might appear";
-    }
-    qDebug() << "Post minimum height:" << postMinimumHeight;
-    postText->setMinimumHeight(postMinimumHeight);
 
 
     /////////////////////////////////////////////////////// Comments block
@@ -433,20 +443,24 @@ Post::Post(PumpController *pumpController,
             this, SLOT(getAllComments()));
 
 
+    if (!this->postTitle.isEmpty())
+    {
+        rightColumnLayout->addWidget(postTitleLabel);
+    }
     rightColumnLayout->addWidget(postText,      2);
     rightColumnLayout->addLayout(buttonsLayout, 0);
-    rightColumnLayout->addWidget(commenter,     0);
+    rightColumnLayout->addWidget(commenter,     1);
 
 
     // Set the initial likes, comments and shares (4 most recent)
-    this->setLikes(lastLikesVariantList);
+    this->setLikes(lastLikesVariantList, likesCount);
     this->commenter->setComments(lastCommentsVariantList);
-    this->setShares(lastSharesVariantList);
+    this->setShares(lastSharesVariantList, sharesCount);
 
 
     mainLayout = new QHBoxLayout();
-    mainLayout->addLayout(leftColumnLayout,  2); // stretch 2/10
-    mainLayout->addLayout(rightColumnLayout, 8); // stretch 8/10
+    mainLayout->addLayout(leftColumnLayout,  2); // stretch 2/11
+    mainLayout->addLayout(rightColumnLayout, 9); // stretch 9/11
 
     mainLayout->setAlignment(Qt::AlignLeft | Qt::AlignTop);
     this->setLayout(mainLayout);
@@ -455,15 +469,195 @@ Post::Post(PumpController *pumpController,
     // Set read status
     this->setPostUnreadStatus();
 
-
-    qDebug() << "Post created";
+    qDebug() << "Post created" << this->postURL;
 }
 
 
 
 Post::~Post()
 {
-    qDebug() << "Post destroyed";
+    qDebug() << "Post destroyed" << this->postURL;
+}
+
+
+/*
+ * Create the menu shown when clicking the post author's avatar
+ *
+ */
+void Post::createAvatarMenu()
+{
+    this->avatarMenu = new QMenu();
+
+    this->avatarMenuIdAction = new QAction(QIcon::fromTheme("user-identity"),
+                                           postAuthorID, this);
+    avatarMenuIdAction->setDisabled(true);
+    avatarMenu->addAction(avatarMenuIdAction);
+    avatarMenu->addSeparator();
+
+    avatarMenu->addAction(QIcon::fromTheme("internet-web-browser"),
+                          tr("Open %1's profile in web browser")
+                          .arg(this->postAuthorName),
+                          this,
+                          SLOT(openAuthorProfileInBrowser()));
+
+    this->avatarMenuFollowAction = new QAction("*follow/unfollow*", this);
+    if (!postIsOwn) // Only add "follow/unfollow" option, if the post is not ours
+    {
+        avatarMenu->addAction(avatarMenuFollowAction);
+        this->setFollowUnfollow();
+    }
+
+    avatarMenu->addSeparator();
+
+    avatarMenu->addAction(QIcon::fromTheme("internet-web-browser"),
+                          tr("Open post in web browser"),
+                          this,
+                          SLOT(openPostInBrowser()));
+    avatarMenu->addAction(QIcon::fromTheme("edit-copy"),
+                          tr("Copy post link to clipboard"),
+                          this,
+                          SLOT(copyPostUrlToClipboard()));
+}
+
+
+/*
+ * Set the icon and text of the follow/unfollow option of the avatar menu
+ * according to whether we're following that user or not
+ *
+ */
+void Post::setFollowUnfollow()
+{
+    if (this->postAuthorFollowed)
+    {
+        this->avatarMenuFollowAction->setIcon(QIcon::fromTheme("list-remove-user"));
+        this->avatarMenuFollowAction->setText(tr("Stop following"));
+        connect(avatarMenuFollowAction, SIGNAL(triggered()),
+                this, SLOT(unfollowUser()));
+        disconnect(avatarMenuFollowAction, SIGNAL(triggered()),
+                   this, SLOT(followUser()));
+        qDebug() << "post author followed, connecting to UNFOLLOW()" << this->postAuthorID;
+    }
+    else
+    {
+        this->avatarMenuFollowAction->setIcon(QIcon::fromTheme("list-add-user"));
+        this->avatarMenuFollowAction->setText(tr("Follow"));
+        connect(avatarMenuFollowAction, SIGNAL(triggered()),
+                this, SLOT(followUser()));
+        disconnect(avatarMenuFollowAction, SIGNAL(triggered()),
+                   this, SLOT(unfollowUser()));
+        qDebug() << "post author not followed, connecting to FOLLOW()" << this->postAuthorID;
+    }
+}
+
+
+
+/*
+ * Set the post contents, and trigger a vertical resize
+ *
+ */
+void Post::setPostContents()
+{
+    //qDebug() << "Post::setPostContents()" << this->postID;
+
+    QString postImage;
+    int imageWidth;
+    if (!this->postImageURL.isEmpty())
+    {
+        QString imageCachedFilename = MiscHelpers::getCachedImageFilename(postImageURL);
+
+        imageWidth = MiscHelpers::getImageWidth(imageCachedFilename);
+        // if the image is wider than the post space, make it smaller
+        if (imageWidth > this->postWidth)
+        {
+            imageWidth = this->postWidth;
+        }
+
+        if (QFile::exists(imageCachedFilename))
+        {
+            postImage = "<br>"
+                        "<a href=\"image:/" + imageCachedFilename + "\">"
+                        "<img src=\"" + imageCachedFilename + "\" "
+                        "width=\"" + QString("%1").arg(imageWidth) + "\" />"
+                        "</a>"
+                        "<br><br>";
+        }
+        else // use placeholder image while it loads...
+        {
+            postImage = "<br>"
+                        "<div title=\"" + tr("Image is loading...") + "\">"
+                        "<img src=\":/images/image-loading.png\"  />"
+                        "</div>"
+                        "<br><br>";
+        }
+    }
+
+
+    QStringList postTextImageList = MiscHelpers::htmlWithReplacedImages(postOriginalText,
+                                                                        this->postWidth);
+    QString postTextContents = postTextImageList.takeAt(0);
+
+    // Add the text content of the post
+    postText->setHtml(postImage
+                      + postTextContents);
+
+
+    this->setPostHeight();
+}
+
+
+
+/*
+ * Set the height of the post, based on the contents
+ *
+ */
+void Post::setPostHeight()
+{
+    int height = postText->document()->size().toSize().height() + 12; // +12px error margin
+
+    if (height < 80)
+    {
+        height = 80;
+    }
+    // FIXME: this will be based on a percentage of window height, not hardcoded!
+    if (height > 450)  // Don't allow a post to have a minimum height > 450 px
+    {
+        height = 450;  // Scrollbars might appear
+    }
+
+    postText->setMinimumHeight(height);
+    postText->setMaximumHeight(height);
+}
+
+
+
+
+
+/*
+ * Add the URL of an image to the queue of pending-download
+ *
+ * connect() signal/slot if necessary
+ *
+ */
+void Post::enqueueImageForDownload(QString url)
+{
+    if (QFile::exists(MiscHelpers::getCachedImageFilename(url))
+     || pendingImagesList.contains(url))
+    {
+        qDebug() << "Post::enqueueImageForDownload(), "
+                    "Using cached post image, or requested image is pending download...";
+    }
+    else
+    {
+        qDebug() << "Post::enqueueImageForDownload(), "
+                    "post image not cached, downloading" << url;
+
+        // FIXME? multi-connect?
+        connect(pController, SIGNAL(imageStored(QString)),
+                this, SLOT(redrawImages(QString)));
+
+        this->pendingImagesList.append(url);
+        pController->getImage(url);
+    }
 }
 
 
@@ -483,7 +677,7 @@ QString Post::likesURL()
  * of the people who liked the post
  *
  */
-void Post::setLikes(QVariantList likesList)
+void Post::setLikes(QVariantList likesList, int likesCount)
 {
     if (likesList.size() > 0)
     {
@@ -513,7 +707,14 @@ void Post::setLikes(QVariantList likesList)
 
     // TMP/FIXME this can set the number to lower than initially set
     // if called with the "initial" up-to-4 likes list that comes with the post
-    this->setLikesLabel(likesList.size());
+    if (likesCount != -1)
+    {
+        this->setLikesLabel(likesCount); // use the passed likesCount parameter
+    }
+    else
+    {
+        this->setLikesLabel(likesList.size()); // show size of actual names list
+    }
 }
 
 
@@ -610,7 +811,7 @@ QString Post::sharesURL()
  * Update the tooltip for "%NUMBER shares" with names
  *
  */
-void Post::setShares(QVariantList sharesList)
+void Post::setShares(QVariantList sharesList, int sharesCount)
 {
     if (sharesList.size() > 0)
     {
@@ -640,7 +841,14 @@ void Post::setShares(QVariantList sharesList)
 
     // TMP/FIXME this can set the number to lower than initially set
     // if called with the "initial" up-to-4 shares list that comes with the post
-    this->setSharesLabel(sharesList.size());
+    if (sharesCount != -1)
+    {
+        this->setSharesLabel(sharesCount); // use the passed sharesCount parameter
+    }
+    else
+    {
+        this->setSharesLabel(sharesList.size()); // show size of actual names list
+    }
 }
 
 
@@ -651,7 +859,7 @@ void Post::setSharesLabel(int resharesCount)
         if (resharesCount == 1)
         {
             postResharesCountLabel->setText(QString::fromUtf8("\342\231\273 ") // recycle symbol
-                                      + tr("Shared one time"));
+                                      + tr("Shared once"));
         }
         else
         {
@@ -700,20 +908,20 @@ void Post::setPostUnreadStatus()
 
 
 
-
 void Post::resizeEvent(QResizeEvent *event)
 {
-    //postText->adjustSize();
-    //postText->resize(postText->contentsRect().size());
-    //postText->resize(postText->frameSize());
-    //this->adjustSize();
-
-    postText->moveCursor(QTextCursor::Start);
+    this->postWidth = postText->width() - 40; // -40 px to account for a possible scrollbar /FIXME
+    if (postWidth < 50)
+    {
+        postWidth = 50;  // minimum width!
+    }
+    setPostContents();
 
     event->accept();
 }
 
 
+
 /*
  * On mouse click in any part of the post, set it as read
  *
@@ -805,7 +1013,17 @@ void Post::commentOnPost()
 {
     qDebug() << "Commenting on this post";
 
-    this->commenter->setFullMode();
+    QString initialText;
+
+    QString selectedText = this->postText->textCursor().selectedText();
+    if (!selectedText.isEmpty())
+    {
+        initialText = MiscHelpers::quotedText(this->postAuthorName,
+                                              selectedText);
+    }
+    this->commenter->setFullMode(initialText);
+
+    emit commentingOnPost(this->commenter);
 }
 
 
@@ -817,7 +1035,7 @@ void Post::sendComment(QString comment)
 {
     qDebug() << "About to publish this comment:" << comment;
 
-    this->pController->addComment(MiscHelpers::cleanHtml(comment),
+    this->pController->addComment(MiscHelpers::cleanupHtml(comment),
                                   this->postID, this->postType);
 }
 
@@ -850,6 +1068,17 @@ void Post::sharePost()
     }
 }
 
+/*
+ * Set the Publisher in editing mode with this post's contents
+ *
+ */
+void Post::editPost()
+{
+    emit postEditRequested(this->postID,
+                           this->postOriginalText,
+                           this->postTitle);
+}
+
 
 
 /*
@@ -884,7 +1113,7 @@ void Post::openClickedURL(QUrl url)
     if (url.scheme() == "image") // Use our own viewer, or maybe a configured external viewer
     {
         qDebug() << "Opening this image in our own viewer...";
-        ImageViewer *viewer = new ImageViewer(url.toString(), this->postImageTitle);
+        ImageViewer *viewer = new ImageViewer(url.toString(), this->postTitle);
         viewer->show();
     }
     else
@@ -892,8 +1121,6 @@ void Post::openClickedURL(QUrl url)
         qDebug() << "Opening this link in browser";
         QDesktopServices::openUrl(url);
     }
-
-
 }
 
 
@@ -920,3 +1147,102 @@ void Post::showHighlightedURL(QUrl url)
         this->highlightedUrlLabel->hide();
     }
 }
+
+
+
+void Post::openAuthorProfileInBrowser()
+{
+    QDesktopServices::openUrl(this->postAuthorURL);
+}
+
+
+void Post::openPostInBrowser()
+{
+    QDesktopServices::openUrl(this->postURL);
+}
+
+void Post::copyPostUrlToClipboard()
+{
+    QApplication::clipboard()->setText(this->postURL);
+}
+
+
+
+
+void Post::followUser()
+{
+    this->pController->followContact(this->postAuthorID);
+
+    postAuthorFollowed = true; // FIXME: should wait until confirmed by server...
+    this->setFollowUnfollow();
+}
+
+
+void Post::unfollowUser()
+{
+    int confirmation = QMessageBox::question(this, tr("Stop following?"),
+                                             tr("Are you sure you want to stop following %1?")
+                                             .arg(this->postAuthorID),
+                                             tr("&Yes, stop following"), tr("&No"), "", 1, 1);
+
+    if (confirmation == 0)
+    {
+        this->pController->unfollowContact(this->postAuthorID);
+
+        postAuthorFollowed = false; // FIXME: should wait until confirmed by server...
+        this->setFollowUnfollow();
+    }
+}
+
+
+
+/*
+ * Trigger a resizeEvent, which will call
+ * setPostContents() and setPostHeight()
+ *
+ */
+void Post::triggerResize()
+{
+    this->resize(this->width() -1,
+                 this->height() -1);
+}
+
+
+
+/*
+ * Redraw post author's avatar after receiving it
+ *
+ */
+void Post::redrawAvatar(QString avatarUrl, QString avatarFilename)
+{
+    if (avatarUrl == this->postAuthorAvatarUrl)
+    {
+        postAuthorAvatarButton->setIcon(QIcon(QPixmap(avatarFilename).scaled(48, 48,
+                                                                  Qt::KeepAspectRatio,
+                                                                  Qt::SmoothTransformation)));
+
+        disconnect(pController, SIGNAL(avatarStored(QString,QString)),
+                   this, SLOT(redrawAvatar(QString,QString)));
+    }
+}
+
+
+/*
+ * Redraw post contents after receiving downloaded images
+ *
+ */
+void Post::redrawImages(QString imageUrl)
+{
+    if (pendingImagesList.contains(imageUrl))
+    {
+        this->pendingImagesList.removeAll(imageUrl);
+        if (pendingImagesList.isEmpty()) // If there are no more, disconnect
+        {
+            disconnect(pController, SIGNAL(imageStored(QString)),
+                       this, SLOT(redrawImages(QString)));
+        }
+
+        // Trigger resizeEvent(), with setPostContents(), etc.
+        triggerResize();
+    }
+}
