@@ -30,6 +30,14 @@ Publisher::Publisher(PumpController *pumpController,
     connect(pController, SIGNAL(postPublishingFailed()),
             this, SLOT(onPublishingFailed()));
 
+    // After receiving the list of lists, update the "Lists" submenus
+    connect(pController, SIGNAL(listsListReceived(QVariantList)),
+            this, SLOT(updateListsMenus(QVariantList)));
+
+
+    this->editingMode = false; // False unless set from setEditingMode
+                               // after clicking "Edit" in a post
+
     this->setFocusPolicy(Qt::StrongFocus); // To keep the publisher from getting focus by accident
 
 
@@ -43,22 +51,25 @@ Publisher::Publisher(PumpController *pumpController,
 
     mainLayout = new QGridLayout();
 
+    titleLabel = new QLabel(tr("Title:"));
+    titleLabel->setSizePolicy(QSizePolicy::Minimum,
+                              QSizePolicy::Maximum);
+
+    titleLineEdit = new QLineEdit();
+    titleLineEdit->setSizePolicy(QSizePolicy::MinimumExpanding,
+                                 QSizePolicy::Maximum);
+    titleLineEdit->setPlaceholderText(tr("Optional title for the post"));
+    titleLineEdit->setToolTip(tr("Title for the post"));
+
 
     pictureLabel = new QLabel();
     pictureLabel->setScaledContents(true);
     pictureLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
     pictureLabel->hide();
 
-    titleLineEdit = new QLineEdit();
-    titleLineEdit->setSizePolicy(QSizePolicy::MinimumExpanding,
-                                 QSizePolicy::Maximum);
-    titleLineEdit->setPlaceholderText(tr("Title for the picture (optional)"));
-    titleLineEdit->setToolTip(tr("Title for the picture"));
-    titleLineEdit->hide();
 
     pictureInfoLabel = new QLabel();
 
-
     selectPictureButton = new QPushButton(QIcon::fromTheme("insert-image"),
                                           tr("Select Picture..."));
     selectPictureButton->setSizePolicy(QSizePolicy::MinimumExpanding,
@@ -68,12 +79,14 @@ Publisher::Publisher(PumpController *pumpController,
             this, SLOT(findPictureFile()));
     selectPictureButton->hide();
 
+
+
     // Set default pixmap and "picture not set" message
     this->setEmptyPictureData();
     lastUsedDirectory = QDir::homePath();
 
 
-
+    // Composer
     composerBox = new Composer(true); // forPublisher = true
     composerBox->setSizePolicy(QSizePolicy::Minimum,
                                QSizePolicy::Minimum);
@@ -93,11 +106,17 @@ Publisher::Publisher(PumpController *pumpController,
     connect(toFollowersAction, SIGNAL(toggled(bool)),
             this, SLOT(setToFollowers(bool)));
 
+    toSelectorListsMenu = new QMenu(tr("Lists"));
+    toSelectorListsMenu->setDisabled(true); // Disabled until lists are received, if any
+    connect(toSelectorListsMenu, SIGNAL(triggered(QAction*)),
+            this, SLOT(updateToListsFields(QAction*)));
+
     toSelectorMenu = new QMenu("to-menu");
     toSelectorMenu->addAction(toPublicAction);
     toSelectorMenu->addAction(toFollowersAction);
+    toSelectorMenu->addMenu(toSelectorListsMenu);
     toSelectorMenu->addSeparator();
-    toSelectorMenu->addAction(tr("More..."), audienceSelectorTo, SLOT(show()));
+    toSelectorMenu->addAction(tr("People..."), audienceSelectorTo, SLOT(show()));
 
     toSelectorButton = new QPushButton(QIcon::fromTheme("system-users"),
                                      tr("To..."));
@@ -105,6 +124,7 @@ Publisher::Publisher(PumpController *pumpController,
     toSelectorButton->setMenu(toSelectorMenu);
 
 
+
     // CC... menu
     ccPublicAction = new QAction(tr("Public"), this);
     ccPublicAction->setCheckable(true);
@@ -116,11 +136,17 @@ Publisher::Publisher(PumpController *pumpController,
     connect(ccFollowersAction, SIGNAL(toggled(bool)),
             this, SLOT(setCCFollowers(bool)));
 
+    ccSelectorListsMenu = new QMenu(tr("Lists"));
+    ccSelectorListsMenu->setDisabled(true); // Disabled until lists are received, if any
+    connect(ccSelectorListsMenu, SIGNAL(triggered(QAction*)),
+            this, SLOT(updateCcListsFields(QAction*)));
+
     ccSelectorMenu = new QMenu("cc-menu");
     ccSelectorMenu->addAction(ccPublicAction);
     ccSelectorMenu->addAction(ccFollowersAction);
+    ccSelectorMenu->addMenu(ccSelectorListsMenu);
     ccSelectorMenu->addSeparator();
-    ccSelectorMenu->addAction(tr("More..."), audienceSelectorCC, SLOT(show()));
+    ccSelectorMenu->addAction(tr("People..."), audienceSelectorCC, SLOT(show()));
 
     ccSelectorButton = new QPushButton(QIcon::fromTheme("system-users"),
                                      tr("CC..."));
@@ -166,6 +192,8 @@ Publisher::Publisher(PumpController *pumpController,
 
     // Sending status info label
     statusInfoLabel = new QLabel();
+    statusInfoLabel->setSizePolicy(QSizePolicy::MinimumExpanding,
+                                   QSizePolicy::Maximum);
     statusInfoLabel->setAlignment(Qt::AlignCenter);
     statusInfoLabel->setWordWrap(true);
     audienceLabelsFont.setBold(false);
@@ -187,32 +215,33 @@ Publisher::Publisher(PumpController *pumpController,
             composerBox, SLOT(cancelPost()));
 
 
-    // Now the layout, starting with "picture mode" stuff
-    mainLayout->addWidget(pictureLabel,        0, 0, 2, 3);
-    mainLayout->addWidget(titleLineEdit,       0, 3, 1, 3, Qt::AlignTop);
-    mainLayout->addWidget(pictureInfoLabel,    1, 3, 1, 1, Qt::AlignBottom);
-    mainLayout->addWidget(selectPictureButton, 1, 4, 1, 2, Qt::AlignBottom | Qt::AlignRight);
+    // Now the layout, starting with the Title field and "picture mode" stuff
+    mainLayout->addWidget(titleLabel,          0, 0, 1, 1, Qt::AlignTop);
+    mainLayout->addWidget(titleLineEdit,       0, 1, 1, 5, Qt::AlignTop);
+    mainLayout->addWidget(pictureLabel,        0, 6, 2, 2);
+    mainLayout->addWidget(pictureInfoLabel,    1, 0, 1, 2, Qt::AlignBottom);
+    mainLayout->addWidget(selectPictureButton, 1, 2, 1, 3, Qt::AlignBottom | Qt::AlignRight);
 
     // The rest depends on the publisher type
     switch (publisherType)
     {
     case 1: // second layout, buttons around
-        mainLayout->addWidget(toSelectorButton, 2, 0, 1, 1);
-        mainLayout->addWidget(ccSelectorButton, 3, 0, 1, 1);
-        mainLayout->addWidget(pictureButton,    4, 0, 1, 1);
-        mainLayout->addWidget(composerBox,      2, 1, 3, 4); // 3 rows, 4 columns
-        mainLayout->addWidget(postButton,       2, 5, 3, 1); // 3 rows
-        postButton->setSizePolicy(QSizePolicy::Maximum,
+        mainLayout->addWidget(toSelectorButton, 2, 0, 1, 1, Qt::AlignTop);
+        mainLayout->addWidget(ccSelectorButton, 3, 0, 1, 1, Qt::AlignTop);
+        mainLayout->addWidget(pictureButton,    4, 0, 1, 1, Qt::AlignTop);
+        mainLayout->addWidget(composerBox,      2, 1, 3, 6); // 3 rows, 6 columns
+        mainLayout->addWidget(postButton,       4, 7, 3, 1); // 3 rows
+        postButton->setSizePolicy(QSizePolicy::MinimumExpanding,
                                   QSizePolicy::MinimumExpanding);
         break;
 
     case 2: // Third layout, buttons on right side
-        mainLayout->addWidget(composerBox,      2, 0, 3, 4); // 3 rows, 4 columns
-        mainLayout->addWidget(toSelectorButton, 2, 4, 1, 1);
-        mainLayout->addWidget(ccSelectorButton, 3, 4, 1, 1);
-        mainLayout->addWidget(pictureButton,    4, 4, 1, 1);
-        mainLayout->addWidget(postButton,       2, 5, 3, 1); // 3 rows, 1 column
-        postButton->setSizePolicy(QSizePolicy::Maximum,
+        mainLayout->addWidget(composerBox,      2, 0, 3, 6); // 3 rows, 6 columns
+        mainLayout->addWidget(toSelectorButton, 2, 6, 1, 1, Qt::AlignTop);
+        mainLayout->addWidget(ccSelectorButton, 3, 6, 1, 1, Qt::AlignTop);
+        mainLayout->addWidget(pictureButton,    4, 6, 1, 1, Qt::AlignTop);
+        mainLayout->addWidget(postButton,       4, 7, 2, 1);
+        postButton->setSizePolicy(QSizePolicy::MinimumExpanding,
                                   QSizePolicy::MinimumExpanding);
         break;
 
@@ -221,11 +250,11 @@ Publisher::Publisher(PumpController *pumpController,
         // just let it jump to default, so incorrect values go to default
 
     default:
-        mainLayout->addWidget(composerBox,      2, 0, 3, 6); // 3 rows, 6 columns
+        mainLayout->addWidget(composerBox,      2, 0, 3, 8); // 3 rows, 8 columns
         mainLayout->addWidget(toSelectorButton, 5, 0, 1, 1, Qt::AlignLeft);
         mainLayout->addWidget(ccSelectorButton, 5, 1, 1, 1, Qt::AlignLeft);
-        mainLayout->addWidget(pictureButton,    5, 3, 1, 1, Qt::AlignLeft);
-        mainLayout->addWidget(postButton,       5, 5, 1, 1);
+        mainLayout->addWidget(pictureButton,    5, 3, 1, 2, Qt::AlignLeft);
+        mainLayout->addWidget(postButton,       5, 7, 1, 1);
 
         break;
 
@@ -240,10 +269,10 @@ Publisher::Publisher(PumpController *pumpController,
     mainLayout->addWidget(ccPublicFollowersLabel, 7, 1, 1, 1);
 
     // the status info label
-    mainLayout->addWidget(statusInfoLabel, 5, 4, 2, 1, Qt::AlignCenter);
+    mainLayout->addWidget(statusInfoLabel, 5, 5, 3, 2, Qt::AlignCenter);
 
     // The "cancel" button
-    mainLayout->addWidget(cancelButton,    7, 5, 1, 1);
+    mainLayout->addWidget(cancelButton,    7, 7, 1, 1);
 
     this->setLayout(mainLayout);
 
@@ -286,7 +315,6 @@ void Publisher::setEmptyPictureData()
                                     Qt::SmoothTransformation));
     pictureLabel->setToolTip(tr("Picture not set"));
 
-    titleLineEdit->clear();
     pictureInfoLabel->clear();
 
     this->pictureFilename.clear();
@@ -297,6 +325,11 @@ void Publisher::setEmptyPictureData()
 void Publisher::updatePublicFollowersLabels()
 {
     QString toString;
+    foreach (QString listName, toListsNameStringList)
+    {
+        toString.append(QString::fromUtf8("\342\236\224 ") // arrow sign in front
+                        + listName + "\n");
+    }
     if (toPublicAction->isChecked())
     {
         toString.append("+" + tr("Public") + "\n");
@@ -309,6 +342,11 @@ void Publisher::updatePublicFollowersLabels()
 
 
     QString ccString;
+    foreach (QString listName, ccListsNameStringList)
+    {
+        ccString.append(QString::fromUtf8("\342\236\224 ") // arrow sign
+                        + listName + "\n");
+    }
     if (ccPublicAction->isChecked())
     {
         ccString.append("+" + tr("Public") + "\n");
@@ -347,6 +385,13 @@ QMap<QString,QString> Publisher::getAudienceMap()
         audienceMap.insertMulti("to|person", "acct:" + address);
     }
 
+    // To: Lists
+    foreach (QString address, this->toListsIdStringList)
+    {
+        audienceMap.insertMulti("to|collection", address);
+    }
+
+
     // To: Followers is checked
     if (toFollowersAction->isChecked())
     {
@@ -366,6 +411,13 @@ QMap<QString,QString> Publisher::getAudienceMap()
         audienceMap.insertMulti("cc|person", "acct:" + address);
     }
 
+    // CC: Lists
+    foreach (QString address, this->ccListsIdStringList)
+    {
+        audienceMap.insertMulti("cc|collection", address);
+    }
+
+
 
     // Last check: if CC: Followers is checked, or nothing is, add Followers
     if (ccFollowersAction->isChecked() || audienceMap.isEmpty())
@@ -397,7 +449,11 @@ void Publisher::setMinimumMode()
     this->composerBox->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
     // ~1 row
-    this->setMaximumHeight((this->composerBox->fontInfo().pixelSize() + 6) * 3);
+    int composerHeight = (this->composerBox->fontInfo().pixelSize() + 4) * 3;
+    this->setMinimumHeight(composerHeight);
+    this->setMaximumHeight(composerHeight);
+    // FIXME: maybe should resize composerBox too/instead
+    //this->composerBox->setMaximumHeight(composerHeight);
 
 
     this->audienceSelectorTo->resetLists();
@@ -409,7 +465,10 @@ void Publisher::setMinimumMode()
     toAudienceLabel->hide();
 
     this->toAddressStringList.clear();
+    this->toListsNameStringList.clear();
+    this->toListsIdStringList.clear();
 
+    toPublicFollowersLabel->clear();
     toPublicFollowersLabel->hide();
 
 
@@ -419,7 +478,10 @@ void Publisher::setMinimumMode()
     ccAudienceLabel->hide();
 
     this->ccAddressStringList.clear();
+    this->ccListsNameStringList.clear();
+    this->ccListsIdStringList.clear();
 
+    ccPublicFollowersLabel->clear();
     ccPublicFollowersLabel->hide();
 
 
@@ -433,10 +495,24 @@ void Publisher::setMinimumMode()
     ccPublicAction->setChecked(false);
     ccFollowersAction->setChecked(true);  // CC: Followers by default
 
+    // Uncheck the lists, very TMP / FIXME
+    // This doesn't clear the To/CC labels sometimes
+    foreach (QAction *action, toSelectorListsMenu->actions())
+    {
+        action->setChecked(false);
+    }
+    foreach (QAction *action, ccSelectorListsMenu->actions())
+    {
+        action->setChecked(false);
+    }
 
+
+    titleLabel->hide();
+    this->titleLineEdit->clear();
+    titleLineEdit->hide();
+    this->statusInfoLabel->clear();
+    statusInfoLabel->hide();
     this->pictureButton->hide();
-    statusInfoLabel->clear();
-    this->statusInfoLabel->hide();
     this->postButton->hide();
     this->cancelButton->hide();
 
@@ -446,11 +522,23 @@ void Publisher::setMinimumMode()
 
     // Hide "picture mode" controls
     this->pictureLabel->hide();
-    this->titleLineEdit->hide();
     this->pictureInfoLabel->hide();
     this->selectPictureButton->hide();
 
     this->setEmptyPictureData();
+
+
+    // Clear "editing mode", restore stuff
+    if (editingMode)
+    {
+        this->editingMode = false;
+        this->editingPostId.clear();
+
+        this->postButton->setText(tr("Post")); // Button text back to "Post" as usual
+        this->toSelectorButton->setEnabled(true);
+        this->ccSelectorButton->setEnabled(true);
+        this->pictureButton->setEnabled(true);
+    }
 }
 
 
@@ -460,6 +548,9 @@ void Publisher::setFullMode()
 {
     qDebug() << "setting Publisher to full mode";
 
+    this->titleLabel->show();
+    this->titleLineEdit->show();
+
     this->composerBox->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
     this->composerBox->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
     this->setMaximumHeight(2048); // i.e. "unlimited"
@@ -473,6 +564,7 @@ void Publisher::setFullMode()
     this->ccAudienceLabel->show();
     this->ccPublicFollowersLabel->show();
 
+    updatePublicFollowersLabels();
 
     // Avoid re-enabling the picture button when re-focusing publisher, but still in picture mode...
     if (pictureButton->isHidden())
@@ -497,7 +589,6 @@ void Publisher::setPictureMode()
 
     this->pictureLabel->show();
 
-    this->titleLineEdit->show();
     this->pictureInfoLabel->show();
     this->selectPictureButton->show();
 
@@ -505,6 +596,43 @@ void Publisher::setPictureMode()
 }
 
 
+/*
+ * Set Publiser to edit mode, after user clicks on "Edit" in a post
+ *
+ */
+void Publisher::setEditingMode(QString postID, QString postText,
+                               QString postTitle)
+{
+    if (postButton->isVisible())  // FIXME, use a proper state variable
+    {
+        // TMPFIX; temporary way of preventing the "Edit" option
+        // from destroying a post currently being composed!
+        QMessageBox::warning(this, tr("Error: Already composing"),
+                             tr("You can't edit a post at this time, "
+                                "because a post is already being composed."));
+        return;
+    }
+
+    this->editingMode = true;
+    this->editingPostId = postID;
+    setFullMode();
+
+
+    // Fill in the contents of the post
+    this->titleLineEdit->setText(postTitle);
+    this->composerBox->setText(postText);
+
+    // Change/disable some controls
+    this->postButton->setText(tr("Update"));
+    this->toSelectorButton->setDisabled(true);
+    this->ccSelectorButton->setDisabled(true);
+    this->pictureButton->setDisabled(true);
+
+
+    this->statusInfoLabel->setText(tr("Editing post"));
+}
+
+
 /*
  * After the post is confirmed to have been received by the server
  * re-enable publisher, clear text, etc.
@@ -528,7 +656,7 @@ void Publisher::onPublishingOk()
 void Publisher::onPublishingFailed()
 {
     qDebug() << "Posting failed, re-enabling Publisher";
-    this->statusInfoLabel->setText(tr("Posting failed.\nTry again."));
+    this->statusInfoLabel->setText(tr("Posting failed.\n\nTry again."));
     this->setEnabled(true);
     this->composerBox->setFocus();
 }
@@ -579,73 +707,6 @@ void Publisher::setCCFollowers(bool activated)
 
 
 
-void Publisher::findPictureFile()
-{
-    QString filename;
-
-    filename = QFileDialog::getOpenFileName(this, tr("Select one image"),
-                                            this->lastUsedDirectory,
-                                            tr("Image files") + "(*.png *.jpg *.jpeg *.gif);;"
-                                            + tr("All files") + " (*)");
-
-    if (!filename.isEmpty())
-    {
-        qDebug() << "Selected" << filename << "for upload";
-        this->pictureLabel->setToolTip(filename);
-
-        QString formatString = QString::fromLocal8Bit(QImageReader::imageFormat(filename));
-        qDebug() << "Image format:" << formatString;
-
-        if (formatString == "png" || formatString == "jpeg" || formatString == "gif")
-        {
-            this->pictureContentType = "image/" + formatString;
-            this->pictureFilename = filename;
-
-            QPixmap imagePixmap = QPixmap(filename);
-            this->pictureLabel->setPixmap(imagePixmap.scaled(200, 150,
-                                                             Qt::IgnoreAspectRatio,
-                                                             Qt::SmoothTransformation));
-
-            QFileInfo fileInfo(filename);
-            int filesize = fileInfo.size();
-            this->lastUsedDirectory = fileInfo.path();
-            qDebug() << "last used directory:" << lastUsedDirectory;
-            QString sizeUnit = tr("bytes");
-            if (filesize > 1024) // if > 1024 bytes, transform to KiB
-            {
-                filesize /= 1024;
-                sizeUnit = "KiB";
-            }
-            if (filesize > 1024) // if > 1024 KiB, transform to MiB
-            {
-                filesize /= 1024;
-                sizeUnit = "MiB";
-            }
-            pictureInfoLabel->setText(QString("<b>" +tr("Resolution") + ":</b> %1x%2"
-                                      "<br>"
-                                      "<b>" + tr("Size") + ":</b> %3 %4")
-                                      .arg(imagePixmap.width())
-                                      .arg(imagePixmap.height())
-                                      .arg(filesize)
-                                      .arg(sizeUnit));
-        }
-        else
-        {
-            // In the future, avoid this by using libmagic or similar
-            qDebug() << "Unknown image format; Extension is probably wrong";
-            QMessageBox::warning(this, tr("Invalid image"),
-                                 tr("The image format cannot be detected.\n"
-                                    "The extension might be wrong, like a GIF "
-                                    "image renamed to image.jpg or similar."));
-
-            this->pictureContentType.clear();
-            this->pictureFilename.clear();
-        }
-    }
-}
-
-
-
 /*
  * Receive a list of addresses for the To or CC fields
  *
@@ -690,6 +751,99 @@ void Publisher::updateToCcFields(QString selectorType,
 
 
 
+/*
+ * Fill the "Lists" submenus with PumpController's lists info
+ *
+ *
+ */
+void Publisher::updateListsMenus(QVariantList listsList)
+{
+    // First, clear the menus
+    toSelectorListsMenu->clear(); // clear() should delete the actions
+    ccSelectorListsMenu->clear();
+
+    if (listsList.length() > 0) // if there are some lists, enable the menu
+    {
+        toSelectorListsMenu->setEnabled(true);
+        ccSelectorListsMenu->setEnabled(true);
+    }
+
+    QString listName;
+    QVariant listId;
+    foreach (QVariant list, listsList)
+    {
+        listName = list.toMap().value("displayName").toString();
+        listId = list.toMap().value("id");
+
+        QAction *toListAction = new QAction(listName, this);
+        toListAction->setCheckable(true);
+        toListAction->setData(listId);
+        toSelectorListsMenu->addAction(toListAction);
+
+        QAction *ccListAction = new QAction(listName, this);
+        ccListAction->setCheckable(true);
+        ccListAction->setData(listId);
+        ccSelectorListsMenu->addAction(ccListAction);
+    }
+}
+
+
+
+void Publisher::updateToListsFields(QAction *listAction)
+{
+    QString listName = listAction->text();
+    QString listId = listAction->data().toString();
+
+    if (listAction->isChecked())
+    {
+        qDebug() << "To list selected" << listName << listId;
+
+        toListsNameStringList.append(listName);
+        toListsIdStringList.append(listId);
+    }
+    else
+    {
+        qDebug() << "To list unselected" << listName;
+
+        toListsNameStringList.removeAll(listName);
+        toListsIdStringList.removeAll(listId);
+    }
+
+    this->updatePublicFollowersLabels();
+
+    qDebug() << "Current To Lists:" << toListsNameStringList << toListsIdStringList;
+}
+
+
+
+void Publisher::updateCcListsFields(QAction *listAction)
+{
+    QString listName = listAction->text();
+    QString listId = listAction->data().toString();
+
+    if (listAction->isChecked())
+    {
+        qDebug() << "CC list selected" << listName << listId;
+
+        ccListsNameStringList.append(listName);
+        ccListsIdStringList.append(listId);
+    }
+    else
+    {
+        qDebug() << "CC list unselected" << listName;
+
+        ccListsNameStringList.removeAll(listName);
+        ccListsIdStringList.removeAll(listId);
+    }
+
+    this->updatePublicFollowersLabels();
+
+    qDebug() << "Current CC Lists:" << ccListsNameStringList << ccListsIdStringList;
+}
+
+
+
+
 /*
  * Send the post (note, image...) to the server
  *
@@ -703,32 +857,106 @@ void Publisher::sendPost()
     if (composerBox->textCursor().document()->characterCount() > 1     // kinda tmp
       || !pictureFilename.isEmpty())
     {
-        QString cleanHtml = MiscHelpers::cleanHtml(composerBox->toHtml());
+        QString postTitle = this->titleLineEdit->text();
+        postTitle.replace("\n", " "); // Post title could have newlines if copy-pasted
+
+        QString cleanHtmlString = MiscHelpers::cleanupHtml(composerBox->toHtml());
 
 
         QMap<QString,QString> audienceMap = this->getAudienceMap();
 
 
-        this->statusInfoLabel->setText(tr("Posting..."));
         // Don't erase just yet!! Just disable until we get "200 OK" from the server.
         this->setDisabled(true);
 
-
-        if (this->pictureFilename.isEmpty())
+        if (!editingMode)
         {
-            this->pController->postNote(audienceMap, cleanHtml);
+            this->statusInfoLabel->setText(tr("Posting..."));
+
+            if (this->pictureFilename.isEmpty())
+            {
+                this->pController->postNote(audienceMap,
+                                            cleanHtmlString,
+                                            postTitle);
+            }
+            else
+            {
+                this->pController->postImage(audienceMap,
+                                             cleanHtmlString,
+                                             postTitle,
+                                             pictureFilename,
+                                             pictureContentType);
+            }
         }
         else
         {
-            this->pController->postImage(audienceMap,
-                                         cleanHtml,
-                                         this->titleLineEdit->text(), // Title
-                                         pictureFilename,
-                                         pictureContentType);
+            this->statusInfoLabel->setText(tr("Updating..."));
+            this->pController->updatePost(this->editingPostId,
+                                          cleanHtmlString,
+                                          postTitle);
         }
     }
     else
     {
+        this->statusInfoLabel->setText(tr("Post is empty."));
         qDebug() << "Can't send post: text is empty, and no picture";
     }
 }
+
+
+
+/*
+ * Let the user find an image file in his/her folders, for picture upload
+ *
+ */
+void Publisher::findPictureFile()
+{
+    QString filename;
+
+    filename = QFileDialog::getOpenFileName(this, tr("Select one image"),
+                                            this->lastUsedDirectory,
+                                            tr("Image files") + "(*.png *.jpg *.jpeg *.gif);;"
+                                            + tr("All files") + " (*)");
+
+    if (!filename.isEmpty())
+    {
+        qDebug() << "Selected" << filename << "for upload";
+        this->pictureLabel->setToolTip(filename);
+
+        this->pictureContentType = MiscHelpers::getImageContentType(filename);
+        if (!pictureContentType.isEmpty())
+        {
+            this->pictureFilename = filename;
+
+            QPixmap imagePixmap = QPixmap(filename);
+            // FIXME: check imagePixmap.isNull() ?
+            this->pictureLabel->setPixmap(imagePixmap.scaled(200, 150,
+                                                             Qt::IgnoreAspectRatio,
+                                                             Qt::SmoothTransformation));
+            QFileInfo fileInfo(filename);
+            this->lastUsedDirectory = fileInfo.path();
+            qDebug() << "last used directory:" << lastUsedDirectory;
+
+
+            pictureInfoLabel->setText(QString("<b>" +tr("Resolution") + ":</b> %1x%2"
+                                      "<br>"
+                                      "<b>" + tr("Size") + ":</b> %3")
+                                      .arg(imagePixmap.width())
+                                      .arg(imagePixmap.height())
+                                      .arg(MiscHelpers::fileSizeString(filename)));
+        }
+        else
+        {
+            // In the future, avoid this by using libmagic or similar
+            qDebug() << "Unknown image format; Extension is probably wrong";
+            QMessageBox::warning(this, tr("Invalid image"),
+                                 tr("The image format cannot be detected.\n"
+                                    "The extension might be wrong, like a GIF "
+                                    "image renamed to image.jpg or similar."));
+
+            this->pictureContentType.clear();
+
+            this->pictureFilename.clear();
+        }
+    }
+}
