@@ -28,7 +28,6 @@
 #include <QPushButton>
 #include <QMenu>
 #include <QFileDialog>
-#include <QImageReader>
 #include <QMessageBox>
 
 #include <QDebug>
@@ -55,7 +54,6 @@ public:
 
     QMap<QString,QString> getAudienceMap();
 
-
 signals:
 
 
@@ -64,6 +62,10 @@ public slots:
     void setFullMode();
     void setPictureMode();
 
+    void setEditingMode(QString postID,
+                        QString postText,
+                        QString postTitle);
+
     void onPublishingOk();
     void onPublishingFailed();
 
@@ -72,20 +74,27 @@ public slots:
     void setCCPublic(bool activated);
     void setCCFollowers(bool activated);
 
-    void findPictureFile();
-
     void updateToCcFields(QString selectorType,
                           QStringList contactsList);
 
+    void updateListsMenus(QVariantList listsList);
+    void updateToListsFields(QAction *listAction);
+    void updateCcListsFields(QAction *listAction);
+
+
     void sendPost();
 
+    void findPictureFile();
+
+
 private:
     QGridLayout *mainLayout;
 
-    QLabel *pictureLabel;
+    QLabel *titleLabel;
     QLineEdit *titleLineEdit;
     QLabel *pictureInfoLabel;
     QPushButton *selectPictureButton;
+    QLabel *pictureLabel;
 
     Composer *composerBox;
 
@@ -93,11 +102,13 @@ private:
     QAction *toPublicAction;
     QAction *toFollowersAction;
     QMenu *toSelectorMenu;
+    QMenu *toSelectorListsMenu;
 
     QPushButton *ccSelectorButton;
     QAction *ccPublicAction;
     QAction *ccFollowersAction;
     QMenu *ccSelectorMenu;
+    QMenu *ccSelectorListsMenu;
 
     AudienceSelector *audienceSelectorTo;
     AudienceSelector *audienceSelectorCC;
@@ -110,6 +121,11 @@ private:
     QStringList toAddressStringList;
     QStringList ccAddressStringList;
 
+    QStringList toListsNameStringList;
+    QStringList toListsIdStringList;
+    QStringList ccListsNameStringList;
+    QStringList ccListsIdStringList;
+
     bool defaultPublicPosting;
 
 
@@ -124,6 +140,8 @@ private:
     QString pictureContentType;
     QString lastUsedDirectory;
 
+    bool editingMode;
+    QString editingPostId;
 
     PumpController *pController;
 };
