@@ -22,11 +22,12 @@
 
 PumpController::PumpController(QObject *parent) :  QObject(parent)
 {
-    this->userAgentString = "Dianara/0.9";
+    this->userAgentString = "Dianara/1.0";
 
     this->postsPerPage = 20;
     this->updatesToTimelineBlocked = false;
 
+
     qoauth = new QOAuth::Interface();
     qoauth->setRequestTimeout(10000); // 10 sec timeout
 
@@ -143,10 +144,12 @@ void PumpController::setUserCredentials(QString userID)
     qDebug() << "New userID is:" << this->userID;
 
 
-    this->getUserProfile(this->userID);
+    // This will call getUserProfile(), getContactList(), getMainTimeline(), etc.
+    this->haveProfile = false;
+    this->haveFollowing = false;
+    this->haveFollowers = false;
+    this->havePersonLists = false;
 
-
-    // This will call getContactList() and getMainTimeline(), etc.
     this->initialDataStep = 0;
     this->initialDataTimer->start(2000);  // start 2 seconds after setting the ID
                                           // (mainly on program startup)
@@ -192,11 +195,12 @@ void PumpController::getUserProfile(QString userID)
 }
 
 
+
 /*
  * Update user's profile
  *
  */
-void PumpController::updateUserProfile(QString avatarURL, QString fullName,
+void PumpController::updateUserProfile(QString avatarUrl, QString fullName,
                                        QString hometown, QString bio)
 {
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/profile";
@@ -206,7 +210,9 @@ void PumpController::updateUserProfile(QString avatarURL, QString fullName,
 
 
     QVariantMap jsonVariantImage;
-    jsonVariantImage.insert("url", avatarURL);
+    jsonVariantImage.insert("url", avatarUrl);
+    jsonVariantImage.insert("width",  90);   // FIXME: don't hardcode this
+    jsonVariantImage.insert("height", 90);   // get values from actual pixmap
 
     QVariantMap jsonVariantLocation;
     jsonVariantLocation.insert("objectType", "place");
@@ -214,19 +220,19 @@ void PumpController::updateUserProfile(QString avatarURL, QString fullName,
 
     QVariantMap jsonVariant;
     jsonVariant.insert("objectType", "person");
-    // TODO:
-    // jsonVariant.insert("image", jsonVariantImage);
+
+    if (!avatarUrl.isEmpty()) // Only add image object if a new image was uploaded
+    {
+        jsonVariant.insert("image", jsonVariantImage);
+    }
     jsonVariant.insert("displayName", fullName);
     jsonVariant.insert("location", jsonVariantLocation);
     jsonVariant.insert("summary", bio);
 
 
-
     QJson::Serializer serializer;
     QByteArray data = serializer.serialize(jsonVariant);
 
-
-
     nam.put(updateProfileRequest, data);
     qDebug() << "Updating user profile" << fullName << hometown;
 }
@@ -268,6 +274,18 @@ void PumpController::getImage(QString imageURL)
 
 
 
+void PumpController::notifyAvatarStored(QString avatarUrl, QString avatarFilename)
+{
+    emit avatarStored(avatarUrl, avatarFilename);
+}
+
+void PumpController::notifyImageStored(QString imageUrl)
+{
+    emit imageStored(imageUrl);
+}
+
+
+
 
 /*
  * GET https://pumpserver.example/api/user/username/following or /followers
@@ -301,6 +319,29 @@ void PumpController::getContactList(QString listType)
 }
 
 
+/*
+ * Check if a userID is in the "following" list
+ *
+ */
+bool PumpController::userInFollowing(QString contactId)
+{
+    if (followingIdList.contains(contactId))
+    {
+        return true;
+    }
+    else
+    {
+        return false;
+    }
+}
+
+
+void PumpController::updateInternalFollowingIdList(QStringList idList)
+{
+    this->followingIdList = idList;
+}
+
+
 
 /*
  * GET https://pumpserver.example/api/user/username/lists/person
@@ -316,12 +357,62 @@ void PumpController::getListsList()
                                                             ListsListRequest,
                                                             QOAuth::ParamMap());
 
-    emit currentJobChanged(tr("Getting list of lists..."));
+    emit currentJobChanged(tr("Getting list of person lists..."));
 
     nam.get(listsListRequest);
 }
 
 
+/*
+ * Create a person list
+ *
+ */
+void PumpController::createPersonList(QString name, QString description)
+{
+    qDebug() << "PumpController() creating person list:" << name;
+
+    QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
+
+    QNetworkRequest postRequest = this->prepareRequest(url, QOAuth::POST, CreatePersonListRequest);
+
+
+    QVariantList jsonVariantObjectTypes;
+    jsonVariantObjectTypes << "person";
+
+    QVariantMap jsonVariantObject;
+    jsonVariantObject.insert("objectType", "collection");
+    jsonVariantObject.insert("objectTypes", jsonVariantObjectTypes);
+    jsonVariantObject.insert("displayName", name);
+    jsonVariantObject.insert("content", description);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "create");
+    jsonVariant.insert("object", jsonVariantObject);
+
+
+    QJson::Serializer serializer;
+    QByteArray data = serializer.serialize(jsonVariant);
+
+    qDebug() << "About to POST:" << data;
+
+
+    emit currentJobChanged(tr("Creating person list..."));
+    nam.post(postRequest, data);
+}
+
+
+void PumpController::deletePersonList(QString id)
+{
+    qDebug() << "PumpController::deletePersonList() deleting list" << id;
+
+    QNetworkRequest deleteRequest = this->prepareRequest(id, QOAuth::DELETE,
+                                                         DeletePersonListRequest);
+
+    emit currentJobChanged(tr("Deleting person list..."));
+    nam.deleteResource(deleteRequest);
+}
+
+
 
 
 /*
@@ -508,7 +599,7 @@ void PumpController::getMinorFeed()
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/inbox/minor";
 
     QOAuth::ParamMap paramMap;
-    paramMap.insert("count", "40");
+    paramMap.insert("count", "50");
 
     QNetworkRequest timelineRequest = this->prepareRequest(url, QOAuth::GET,
                                                            MinorFeedRequest, paramMap);
@@ -565,14 +656,16 @@ QNetworkRequest PumpController::prepareRequest(QString url, QOAuth::HttpMethod m
  * Used to upload pictures, but might have more uses in the future
  *
  */
-void PumpController::uploadFile(QString filename, QString contentType)
+void PumpController::uploadFile(QString filename,
+                                QString contentType,
+                                int uploadType)
 {
     qDebug() << "PumpController::uploadFile()";
 
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/uploads";
 
     QNetworkRequest postRequest = this->prepareRequest(url, QOAuth::POST,
-                                                       UploadFileRequest,
+                                                       uploadType,
                                                        QOAuth::ParamMap(),
                                                        contentType);
 
@@ -683,89 +776,161 @@ void PumpController::requestFinished(QNetworkReply *reply)
     }
 
 
+    QString errorTypeString;
+
     switch (httpCode)
     {
+    // First, handle error codes
     case 503:
-        // if not just and image, it's important, so popup notification
+        errorTypeString = tr("Service Unavailable", "HTTP 503 error string")
+                        +  " (503)";
+
+        // if not just an image, it's important, so popup notification
         if (requestType != ImageRequest && requestType != AvatarRequest)
         {
-            emit showNotification("Service Unavailable (503)\n" + reply->url().toString());
+            emit showNotification(errorTypeString + "\n"
+                                  + reply->url().toString());
         }
         else // just status bar
         {
-            emit currentJobChanged("Service Unavailable (503): " + reply->url().toString());
+            emit currentJobChanged(errorTypeString + ": "
+                                   + reply->url().toString());
         }
 
         qDebug() << "HTTP 503: Service Unavailable.";
         qDebug() << "Data:  " << reply->readAll();
+        reply->close();
+        reply->deleteLater();
         return;
 
     case 500:
-        // if not just and image, it's important, so popup notification
+        errorTypeString = tr("Internal Server Error", "HTTP 500 error string")
+                        + " (500)";
+
+        // if not just an image, it's important, so popup notification
         if (requestType != ImageRequest && requestType != AvatarRequest)
         {
-            emit showNotification("Internal server error (500)\n" + reply->url().toString());
+            emit showNotification(errorTypeString + "\n"
+                                  + reply->url().toString() + "\n\n"
+                                  + reply->readLine());
         }
         else
         {
-            emit currentJobChanged("Internal server error (500): " + reply->url().toString());
+            emit currentJobChanged(errorTypeString + ": "
+                                   + reply->url().toString());
         }
-        qDebug() << "HTTP 500: Internal server error.";
+        qDebug() << "HTTP 500: Internal Server Error.";
         qDebug() << "Data:  " << reply->readAll();
+        reply->close();
+        reply->deleteLater();
         return;
 
 
     case 410:
-        emit currentJobChanged("Gone (410): " + reply->url().toString());
+        errorTypeString = tr("Gone", "HTTP 410 error string")
+                        + " (410)";
+        emit currentJobChanged(errorTypeString + ": "
+                               + reply->url().toString());
         qDebug() << "HTTP 410: Gone.";
         qDebug() << "Data:  " << reply->readAll();
+        reply->close();
+        reply->deleteLater();
         return;
 
     case 404:
-        emit currentJobChanged("Not Found (404): " + reply->url().toString());
+        errorTypeString = tr("Not Found", "HTTP 404 error string")
+                        + " (404)";
+        emit currentJobChanged(errorTypeString + ": "
+                               + reply->url().toString());
         qDebug() << "HTTP 404: Not Found.";
         qDebug() << "Data:  " << reply->readAll();
+        reply->close();
+        reply->deleteLater();
         return;
 
     case 403:
-        emit currentJobChanged("Forbidden (403): " + reply->url().toString());
+        errorTypeString = tr("Forbidden", "HTTP 403 error string")
+                        + " (403)";
+        emit currentJobChanged(errorTypeString + ": "
+                               + reply->url().toString());
         qDebug() << "HTTP 403: Forbidden.";
         qDebug() << "Data:  " << reply->readAll();
+        reply->close();
+        reply->deleteLater();
+        return;
+
+    case 401:
+        errorTypeString = tr("Unauthorized", "HTTP 401 error string")
+                        + " (401)";
+        emit currentJobChanged(errorTypeString + ": "
+                               + reply->url().toString());
+        qDebug() << "HTTP 401: Unauthorized.";
+        qDebug() << "Data:  " << reply->readAll();
+        reply->close();
+        reply->deleteLater();
         return;
 
 
     case 400:
-        // if not just and image, it's important, so popup notification
+        errorTypeString = tr("Bad Request", "HTTP 400 error string")
+                        + " (400)";
+
+        // if not just an image, it's important, so popup notification
         if (requestType != ImageRequest && requestType != AvatarRequest)
         {
-            emit showNotification("Bad Request (400)\n" + reply->url().toString());
+            emit showNotification(errorTypeString + "\n"
+                                  + reply->url().toString() + "\n\n"
+                                  + reply->readLine());
         }
         else
         {
-            emit currentJobChanged("Bad Request (400): " + reply->url().toString());
+            emit currentJobChanged(errorTypeString + ": "
+                                   + reply->url().toString());
         }
         qDebug() << "HTTP 400: Bad Request.";
         qDebug() << "Data:  " << reply->readAll();
+        reply->close();
+        reply->deleteLater();
+        return;
+
+    case 302:
+        errorTypeString = tr("Moved Temporarily", "HTTP 302 error string")
+                        + " (302)";
+        emit currentJobChanged(errorTypeString + ": "
+                               + reply->url().toString());
+        qDebug() << "HTTP 302: Moved Temporarily.";
+        qDebug() << "Data:  " << reply->readAll();
+        reply->close();
+        reply->deleteLater();
         return;
 
     case 301:
-        emit currentJobChanged("Moved Permanently (301): " + reply->url().toString());
+        errorTypeString = tr("Moved Permanently", "HTTP 301 error string")
+                        + " (301)";
+        emit currentJobChanged(errorTypeString + ": "
+                               + reply->url().toString());
         qDebug() << "HTTP 301: Moved Permanently.";
         qDebug() << "Data:  " << reply->readAll();
+        reply->close();
+        reply->deleteLater();
         return;
 
     case 0:
-        emit currentJobChanged(tr("Error connecting to %1").arg(reply->url().host()) + ": " + reply->errorString());
-        qDebug() << "Error connecting to" << reply->url().host();
+        emit currentJobChanged(tr("Error connecting to %1").arg(reply->url().host())
+                               + ": " + reply->errorString());
+        qDebug() << "Error connecting to" << reply->url();
+        reply->close();
+        reply->deleteLater();
         return;
 
     // Other HTTP codes
     default:
-        qDebug() << "Unhandled HTTP error " << httpCode;
-        qDebug() << "Data:  " << reply->readAll();
         emit currentJobChanged(tr("Unhandled HTTP error code %1").arg(httpCode)
                               + ": " + reply->url().toString());
-
+        qDebug() << "Unhandled HTTP error " << httpCode;
+        qDebug() << "Data:  " << reply->readAll();
+        reply->close();
+        reply->deleteLater();
         return;
 
 
@@ -828,31 +993,40 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
     case UserProfileRequest:
         qDebug() << "A user profile was requested";
+        emit currentJobChanged(tr("Profile received."));
 
         if (jsonParsedOK && jsonData.size() > 0)
         {
             QVariantMap profileMap = jsonData["profile"].toMap();
 
-            if (profileMap["id"].toString() == "acct:" + this->userID)
+            if (profileMap.value("id").toString() == "acct:" + this->userID)
             {
                 qDebug() << "Received OWN profile";
+                this->haveProfile = true;
 
-                QString profImageUrl = profileMap["image"].toMap()["url"].toString();
-                QString profDisplayName = profileMap["displayName"].toString();
-                QString profLocation = profileMap["location"].toMap()["displayName"].toString();
-                QString profSummary = profileMap["summary"].toString();
+                QString profImageUrl = profileMap.value("image").toMap().value("url").toString();
+                QString profDisplayName = profileMap.value("displayName").toString();
+                QString profLocation = profileMap.value("location").toMap().value("displayName").toString();
+                QString profSummary = profileMap.value("summary").toString();
 
                 emit profileReceived(profImageUrl, profDisplayName,
                                      profLocation, profSummary);
 
                 // Store also the user's followers URL, for posting to Followers
-                this->userFollowersURL = profileMap["followers"].toMap()["url"].toString();
+                this->userFollowersURL = profileMap.value("followers").toMap().value("url").toString();
             }
         }
 
         break;
 
 
+    case UpdateProfileRequest:
+        emit currentJobChanged(tr("Profile updated."));
+        this->getUserProfile(this->userID);
+
+        break;
+
+
 
 
     ////////////////////////////////////////////////// If a timeline was requested
@@ -917,10 +1091,10 @@ void PumpController::requestFinished(QNetworkReply *reply)
         {
             qDebug() << "JSON parsed OK";
 
-            QString objectID = jsonData["object"].toMap()["id"].toString();
+            QString objectID = jsonData["object"].toMap().value("id").toString();
             qDebug() << "Image post ID:" << objectID;
 
-            if (jsonData["object"].toMap()["objectType"].toString() == "image")
+            if (jsonData["object"].toMap().value("objectType").toString() == "image")
             {
                 // Update the image with title and description
                 this->postImageStepThree(objectID);
@@ -941,9 +1115,37 @@ void PumpController::requestFinished(QNetworkReply *reply)
         break;
 
 
+
+    case PublishAvatarRequest:
+        emit currentJobChanged(tr("Avatar published successfully."));
+        if (jsonParsedOK && jsonData.size() > 0)
+        {
+            qDebug() << "JSON parsed OK";
+
+            QString imageUrl = jsonData["object"].toMap().value("image").toMap().value("url").toString();
+            qDebug() << "Avatar post ID:" << imageUrl;
+            // FIXME: get "pump_io: fullImage: url" too
+
+            if (jsonData["object"].toMap().value("objectType").toString() == "image")
+            {
+                emit avatarUploaded(imageUrl);
+            }
+            else
+            {
+                qDebug() << "Avatar uploaded, but type is not IMAGE!";
+            }
+        }
+        else
+        {
+            qDebug() << "Error parsing received JSON data!";
+            qDebug() << "Raw data:" << packetData; // JSON directly
+        }
+
+        break;
+
+
     ////////////////////////////////////
     case UpdatePostRequest:
-        qDebug() << "**************** POST UPDATED OK!";
         emit currentJobChanged(tr("Post updated successfully."));
         emit postPublished();
         break;
@@ -983,7 +1185,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
     ///////////////////////////////////// If commenting on a post was requested
     case CommentPostRequest:
         emit currentJobChanged(tr("Comment posted successfully."));
-        emit commentPosted(); // This will be catched by Commenter()
+        emit commentPosted(); // This will be caught by Commenter()
 
         break;
 
@@ -1033,7 +1235,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
             jsonDataList = jsonData["items"].toList();
             qDebug() << "Number of items in minor feed:" << jsonDataList.size();
 
-             emit minorFeedReceived(jsonDataList);
+            emit minorFeedReceived(jsonDataList);
         }
         else
         {
@@ -1082,11 +1284,13 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
                 if (requestType == FollowingListRequest)
                 {
+                    this->haveFollowing = true;
                     emit currentJobChanged(tr("List of 'following' received."));
                     emit contactListReceived("following", contactsVariant.toList());
                 }
                 else // == FollowersListRequest
                 {
+                    this->haveFollowers = true;
                     emit currentJobChanged(tr("List of 'followers' received."));
                     emit contactListReceived("followers", contactsVariant.toList());
                 }
@@ -1107,7 +1311,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
     case ListsListRequest:
-        qDebug() << "The list of lists was requested";
+        qDebug() << "The list of person lists was requested";
 
         if (jsonParsedOK && jsonData.size() > 0)
         {
@@ -1118,6 +1322,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
             {
                 qDebug() << "Parsed a List, listing lists...";
 
+                this->havePersonLists = true;
                 emit currentJobChanged(tr("List of 'lists' received."));
                 emit listsListReceived(listsVariant.toList());
             }
@@ -1135,6 +1340,19 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
         break;
 
+    // Person list created OK
+    case CreatePersonListRequest:
+        emit currentJobChanged(tr("Person list created successfully."));
+
+        this->getListsList(); // And reload the person lists
+        break;
+
+    // Person list created OK
+    case DeletePersonListRequest:
+        emit currentJobChanged(tr("Person list deleted successfully."));
+
+        this->getListsList(); // And reload the person lists
+        break;
 
 
     case AvatarRequest:
@@ -1166,13 +1384,16 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
     //////////////////////////////////////// If uploading a file was requested
+    case UploadAvatarRequest:
+        // just jump to next
+    case UploadPictureRequest:
+        // just jump
     case UploadFileRequest:
         qDebug() << "Uploading a file was requested";
 
         if (jsonParsedOK && jsonData.size() > 0)
         {
             qDebug() << "JSON parsed OK";
-            emit currentJobChanged(tr("File uploaded successfully. Posting message..."));
 
             QString uploadedFileID = jsonData["id"].toString();
             qDebug() << "Uploaded file ID:" << uploadedFileID;
@@ -1180,7 +1401,16 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
             if (jsonData["objectType"].toString() == "image")
             {
-                this->postImageStepTwo(uploadedFileID);
+                if (requestType == UploadPictureRequest)
+                {
+                    emit currentJobChanged(tr("File uploaded successfully. Posting message..."));
+                    this->postImageStepTwo(uploadedFileID);
+                }
+                else if (requestType == UploadAvatarRequest)
+                {
+                    emit currentJobChanged(tr("Avatar uploaded."));
+                    this->postAvatarStepTwo(uploadedFileID);
+                }
             }
         }
         else
@@ -1194,6 +1424,10 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
     }
     // end switch (requestType)
+
+    qDebug() << "requestFinished() ended; reply->deleteLater() " << reply->url();
+    reply->close();
+    reply->deleteLater();
 }
 
 
@@ -1201,6 +1435,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
 
+
 void PumpController::sslErrorsHandler(QNetworkReply *reply, QList<QSslError> errorList)
 {
     qDebug() << "\n==== SSL errors!! ====";
@@ -1255,7 +1490,8 @@ void PumpController::getToken()
         qDebug() << "About to POST:" << data;
 
 
-        nam.post(postRequest, data); // upon receiving data (id+secret), will execute getToken() again
+        // upon receiving data (id+secret), will execute getToken() again
+        nam.post(postRequest, data);
     }
     else
     {
@@ -1387,7 +1623,8 @@ void PumpController::authorizeApplication(QString verifierCode)
 
 
 /*
- * Called by a QTimer, get initial data (contacts, timeline...) one step at a time
+ * Called by a QTimer, get initial data (profile, contacts, timelines),
+ * one step at a time
  *
  */
 void PumpController::getInitialData()
@@ -1396,42 +1633,56 @@ void PumpController::getInitialData()
 
     initialDataTimer->setInterval(3000);  // Every 3 sec
 
+    /*
+     * FIXME: this needs to be way more elaborate.
+     *
+     * Ensure certain stuff has been received before continuing.
+     *
+     * For instance, getting the "following" list is needed before
+     * being able to post, and to know the state of following/not Following
+     * the author of a post in a timeline
+     *
+     */
 
     switch (this->initialDataStep)
     {
     case 0:
-        this->getMainTimeline(0);
+        this->getUserProfile(this->userID);
         break;
 
     case 1:
-        this->getDirectTimeline(0);
-        break;
-
-    case 2:
-        this->getActivityTimeline(0);
-        break;
-
-    case 3:
-        this->getFavoritesTimeline(0);
-        break;
-
-    case 4:
-        this->getMinorFeed();
-        break;
-
-    case 5:
         this->getContactList("following");
         break;
 
-    case 6:
+    case 2:
         this->getContactList("followers");
         break;
 
-    case 7:
+    case 3:
+        this->getMainTimeline(0);
+        break;
+
+    case 4:
         this->getListsList();
         break;
 
+    case 5:
+        this->getMinorFeed();
+        break;
+
+    case 6:
+        this->getDirectTimeline(0);
+        break;
+
+    case 7:
+        this->getActivityTimeline(0);
+        break;
+
     case 8:
+        this->getFavoritesTimeline(0);
+        break;
+
+    case 9:
         // Do nothing, so it takes longer for the final (default) step to arrive
         break;
 
@@ -1457,7 +1708,7 @@ void PumpController::getInitialData()
  *
  */
 void PumpController::postNote(QMap<QString, QString> audienceMap,
-                              QString postText)
+                              QString postText, QString postTitle)
 {
     qDebug() << "PumpController::postNote()";
 
@@ -1472,6 +1723,10 @@ void PumpController::postNote(QMap<QString, QString> audienceMap,
 
     QVariantMap jsonVariantObject;
     jsonVariantObject.insert("objectType", "note");
+    if (!postTitle.isEmpty())
+    {
+        jsonVariantObject.insert("displayName", postTitle);
+    }
     jsonVariantObject.insert("content", postText);
 
     QVariantMap jsonVariant;
@@ -1517,7 +1772,7 @@ void PumpController::postImage(QMap<QString, QString> audienceMap,
     this->currentAudienceMap = audienceMap;
 
 
-    this->uploadFile(imageFilename, contentType);
+    this->uploadFile(imageFilename, contentType, UploadPictureRequest);
 }
 
 
@@ -1572,7 +1827,10 @@ void PumpController::postImageStepThree(QString id)
 
     QVariantMap jsonVariant;
     jsonVariant.insert("verb", "update"); // "post" would suffice
-    jsonVariant.insert("displayName", this->currentImageTitle);
+    if (!this->currentImageTitle.isEmpty())
+    {
+        jsonVariant.insert("displayName", this->currentImageTitle);
+    }
     jsonVariant.insert("content", this->currentImageDescription);
 
     QJson::Serializer serializer;
@@ -1584,6 +1842,75 @@ void PumpController::postImageStepThree(QString id)
 
 
 
+/*
+ * Second step for avatar upload.
+ *
+ * Post the image to Public
+ *
+ */
+void PumpController::postAvatarStepTwo(QString id)
+{
+        qDebug() << "PumpController::postAvatarStepTwo() image ID:" << id;
+
+        QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
+
+        QNetworkRequest postRequest = this->prepareRequest(url, QOAuth::POST, PublishAvatarRequest);
+
+
+        QVariantMap jsonVariantObject;
+        jsonVariantObject.insert("objectType", "image");
+        jsonVariantObject.insert("id", id);
+
+        // audience, CC: Public
+        QVariantMap jsonVariantPublic;
+        jsonVariantPublic.insert("objectType", "collection");
+        jsonVariantPublic.insert("id", "http://activityschema.org/collection/public");
+
+        QVariantList jsonVariantAudience;
+        jsonVariantAudience.append(jsonVariantPublic);
+
+        QVariantMap jsonVariant;
+        jsonVariant.insert("verb", "post");
+        jsonVariant.insert("object", jsonVariantObject);
+        jsonVariant.insert("cc", jsonVariantAudience); // CC: Public
+
+        QJson::Serializer serializer;
+        QByteArray data = serializer.serialize(jsonVariant);
+
+        qDebug() << "About to POST:" << data;
+
+        nam.post(postRequest, data);
+}
+
+
+
+/*
+ * Update a post contents (the object)
+ *
+ */
+void PumpController::updatePost(QString id,
+                                QString content,
+                                QString title)
+{
+    qDebug() << "PumpController::updatePost(), post ID:" << id;
+
+    // Using the ID directly as URL
+    QNetworkRequest postRequest = this->prepareRequest(id, QOAuth::PUT, UpdatePostRequest);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "update"); // "post" would suffice
+    jsonVariant.insert("displayName", title); // "title" can be empty, as a way to remove titles
+    jsonVariant.insert("content", content);
+
+    QJson::Serializer serializer;
+    QByteArray data = serializer.serialize(jsonVariant);
+
+    qDebug() << "About to PUT:" << data;
+    nam.put(postRequest, data);
+}
+
+
+
 /*
  * Like (favorite) a post, by its ID (URL)
  *
