@@ -69,14 +69,21 @@ public:
     QString currentFollowersURL();
 
     void getUserProfile(QString userID);
-    void updateUserProfile(QString avatarURL, QString fullName,
+    void updateUserProfile(QString avatarUrl, QString fullName,
                            QString hometown, QString bio);
 
     void getAvatar(QString avatarURL);
     void getImage(QString imageURL);
+    void notifyAvatarStored(QString avatarUrl, QString avatarFilename);
+    void notifyImageStored(QString imageUrl);
 
     void getContactList(QString listType);
+    bool userInFollowing(QString contactId);
+    void updateInternalFollowingIdList(QStringList idList);
+
     void getListsList();
+    void createPersonList(QString name, QString description);
+    void deletePersonList(QString id);
 
     void getMainTimeline(int timelineOffset);
     void getDirectTimeline(int timelineOffset);
@@ -95,7 +102,8 @@ public:
                                    QOAuth::ParamMap paramMap = QOAuth::ParamMap(),
                                    QString contentTypeString="application/json");
 
-    void uploadFile(QString filename, QString contentType);
+    void uploadFile(QString filename, QString contentType,
+                    int uploadType = UploadFileRequest);
 
     QList<QVariantList> processAudience(QMap<QString,QString> audienceMap);
 
@@ -112,6 +120,8 @@ public:
         FollowingListRequest,
         FollowersListRequest,
         ListsListRequest,
+        CreatePersonListRequest,
+        DeletePersonListRequest,
 
         MainTimelineRequest,
         DirectTimelineRequest,
@@ -138,7 +148,10 @@ public:
         AvatarRequest,
         ImageRequest,
 
-        UploadFileRequest
+        UploadFileRequest,
+        UploadPictureRequest,
+        UploadAvatarRequest,
+        PublishAvatarRequest
     };
 
 
@@ -153,6 +166,7 @@ signals:
     void contactListReceived(QString listType, QVariantList contactList);
     void listsListReceived(QVariantList listsList);
 
+
     void mainTimelineReceived(QVariantList postList, int postsPerPage,
                               QString previousLink, QString nextLink);
     void directTimelineReceived(QVariantList postList, int postsPerPage,
@@ -169,6 +183,8 @@ signals:
 
     void avatarPictureReceived(QByteArray pictureData, QUrl pictureURL);
     void imageReceived(QByteArray pictureData, QUrl pictureURL);
+    void avatarStored(QString avatarUrl, QString avatarFilename);
+    void imageStored(QString imageUrl);
 
     void postPublished();
     void postPublishingFailed();
@@ -176,6 +192,8 @@ signals:
     void commentPosted();
     void commentPostingFailed();
 
+    void avatarUploaded(QString url);
+
     void showNotification(QString message);
     void currentJobChanged(QString message);
 
@@ -192,13 +210,19 @@ public slots:
     void getInitialData();
 
     void postNote(QMap<QString,QString> audienceMap,
-                  QString postText);
+                  QString postText,
+                  QString postTitle);
     void postImage(QMap<QString,QString> audienceMap,
                    QString postText, QString currentImageTitle,
                    QString imageFilename, QString contentType);
     void postImageStepTwo(QString id);
     void postImageStepThree(QString id);
 
+    void postAvatarStepTwo(QString id);
+
+
+    void updatePost(QString id, QString content, QString title);
+
     void likePost(QString postID, QString postType, bool like);
     void addComment(QString comment, QString postID, QString postType);
     void sharePost(QString postID, QString postType);
@@ -213,12 +237,6 @@ private:
     QNetworkAccessManager nam;
     QByteArray userAgentString;
 
-    QString userID;  // Full webfinger address, user@host.tld
-    QString userName;
-    QString serverURL;
-
-    QString userFollowersURL;
-
     // QOAuth-related
     QOAuth::Interface *qoauth;
 
@@ -231,14 +249,26 @@ private:
     QByteArray tokenSecret;
 
 
+    QString userID;  // Full webfinger address, user@host.tld
+    QString userName;
+    QString serverURL;
+
+    QString userFollowersURL;
+    QStringList followingIdList;
+
     QTimer *initialDataTimer;
     int initialDataStep;
+    bool haveProfile;
+    bool haveFollowing;
+    bool haveFollowers;
+    bool havePersonLists;
 
 
     int postsPerPage;
     bool updatesToTimelineBlocked;
 
 
+    // For multi-step operations in posts
     QString currentImageTitle;
     QString currentImageDescription;
     QMap<QString, QString> currentAudienceMap;
