@@ -25,13 +25,8 @@ TimeLine::TimeLine(int timelineType,
                    QWidget *parent) :  QWidget(parent)
 {
     this->timelineType = timelineType;
-
     this->pController = pumpController;
 
-    mainLayout = new QVBoxLayout();
-    mainLayout->setAlignment(Qt::AlignTop);
-
-
     QMap<QString,QString> demoPostData;
     demoPostData.insert("author",         "Demo User");
     demoPostData.insert("authorId",       "demo@somepump.example");
@@ -48,9 +43,8 @@ TimeLine::TimeLine(int timelineType,
     switch (this->timelineType)
     {
     case TimelineTypeMain:
-        demoPostData.insert("text",   "<h2><b>" +tr("Welcome to Dianara") + "</b></h2><br>"
-
-                                      + tr("Dianara is a <b>pump.io</b> client.")
+        demoPostData.insert("title",  tr("Welcome to Dianara"));
+        demoPostData.insert("text",   tr("Dianara is a <b>pump.io</b> client.")
                                       + "<br>"
 
                                       + tr("If you don't have a Pump account yet, you can get one "
@@ -88,31 +82,25 @@ TimeLine::TimeLine(int timelineType,
 
 
     case TimelineTypeDirect:
-        demoPostData.insert("text",   "<h2><b>" + tr("Direct Messages Timeline") + "</b></h2><br>"
-
-                                      + tr("Here, you'll see posts specifically directed to you.")
-
+        demoPostData.insert("title",  tr("Direct Messages Timeline"));
+        demoPostData.insert("text",   tr("Here, you'll see posts specifically directed to you.")
                                       + "<br>");
         this->previousNewestPostID = settings.value("previousNewestPostIDDirect", "").toString();
         break;
 
 
     case TimelineTypeActivity:
-        demoPostData.insert("text",   "<h2><b>" + tr("Activity Timeline") + "</b></h2><br>"
-
-                                      + tr("You'll see your own posts here.")
-
-                                      +"<br>");
+        demoPostData.insert("title", tr("Activity Timeline"));
+        demoPostData.insert("text",  tr("You'll see your own posts here.")
+                                     +"<br>");
         this->previousNewestPostID = settings.value("previousNewestPostIDActivity", "").toString();
         break;
 
 
     case TimelineTypeFavorites:
-        demoPostData.insert("text",   "<h2><b>" + tr("Favorites Timeline") + "</b></h2><br>"
-
-                                      + tr("Posts and comments you've liked.")
-
-                                      + "<br>");
+        demoPostData.insert("title", tr("Favorites Timeline"));
+        demoPostData.insert("text", tr("Posts and comments you've liked.")
+                                    + "<br>");
         this->previousNewestPostID = settings.value("previousNewestPostIDFavorites", "").toString();
         break;
 
@@ -126,10 +114,6 @@ TimeLine::TimeLine(int timelineType,
 
 
 
-    splitter = new QSplitter(Qt::Vertical, this);
-    splitter->setChildrenCollapsible(false);
-    mainLayout->addWidget(splitter);
-
 
     this->unreadPostsCount = 0;
     this->timelineOffset = 0;
@@ -151,6 +135,15 @@ TimeLine::TimeLine(int timelineType,
             this, SLOT(goToNextPage()));
 
 
+    ///// Layout
+    mainLayout = new QVBoxLayout();
+
+    postsLayout = new QVBoxLayout();
+    postsLayout->setAlignment(Qt::AlignTop);
+    mainLayout->addLayout(postsLayout, 1);
+
+    mainLayout->addSpacing(2); // 2 pixel separation
+
     bottomLayout = new QHBoxLayout();
     bottomLayout->addWidget(firstPageButton);
     bottomLayout->addSpacing(8);
@@ -158,18 +151,18 @@ TimeLine::TimeLine(int timelineType,
     bottomLayout->addSpacing(8);
     bottomLayout->addWidget(previousPageButton);
     bottomLayout->addWidget(nextPageButton);
-    mainLayout->addLayout(bottomLayout);
-
-
-    // Add the default "demo" post
-    splitter->addWidget(new Post(pController, demoPostData,
-                                 QVariantList(), QVariantList(), QVariantList(),
-                                 this));
-
+    mainLayout->addLayout(bottomLayout, 0);
 
     this->setLayout(mainLayout);
 
 
+    // Add the default "demo" post
+    Post *demoPost = new Post(pController, demoPostData,
+                              QVariantList(), QVariantList(), QVariantList(),
+                              this);
+    postsInTimeline.append(demoPost);
+    postsLayout->addWidget(demoPost);
+
     qDebug() << "TimeLine created";
 }
 
@@ -211,33 +204,11 @@ TimeLine::~TimeLine()
  */
 void TimeLine::clearTimeLineContents()
 {
-    QList<QWidget*> previousPostList;
-
-    // Build list of widgets contained in the layout
-    for (int counter = 0; counter < splitter->count(); ++counter)
+    foreach (Post *oldPost, postsInTimeline)
     {
-        //qDebug() << "Adding for deletion:" << splitter->widget(counter);
-        previousPostList.append(splitter->widget(counter));
-    }
-
-    qDebug() << "List to delete:" << previousPostList;
-
-
-    foreach (QWidget *oldPost, previousPostList)
-    {
-        qDebug() << "will deleteLater:" << oldPost;
         oldPost->deleteLater(); // delete should be safe too, but still...
     }
-
     this->postsInTimeline.clear();
-
-    /*
-     * This is to avoid the splitter getting bigger and bigger every time the user
-     * updates the timeline (F5, etc)
-     *
-     */
-    this->splitter->setMinimumHeight(0);
-    this->splitter->resize(1,1);
 }
 
 
@@ -279,6 +250,21 @@ void TimeLine::updateCurrentPageNumber()
 }
 
 
+/*
+ * Resize all posts in timeline
+ *
+ */
+void TimeLine::resizePosts()
+{
+    foreach (Post *post, postsInTimeline)
+    {
+        // Force a Post() resizeEvent, which will call
+        // setPostContents() and setPostHeight()
+        post->resize(post->width() - 1,
+                     post->height() - 1);
+    }
+}
+
 
 
 /*********************************************************/
@@ -318,7 +304,7 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             QVariantMap objectMap;
             if (this->timelineType != TimelineTypeFavorites)  // VERY TMP!! Favorites-tmp-stuff
             {
-                objectMap = singlePostMap["object"].toMap();
+                objectMap = singlePostMap.value("object").toMap();
             }
             else
             {
@@ -326,11 +312,11 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
                 objectMap = singlePostMap;
             }
 
-
-            QString itemVerb = singlePostMap["verb"].toString();
+            // Activity verb: post, share...
+            QString itemVerb = singlePostMap.value("verb").toString();
 
             // This will hold the date when the post was deleted, or empty if it wasn't
-            QString postIsDeleted = objectMap["deleted"].toString();
+            QString postIsDeleted = objectMap.value("deleted").toString();
 
 
             QString postID;
@@ -339,16 +325,18 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             QString postAuthor;
             QString postAuthorAvatar;
             QString postAuthorId;
+            QString postAuthorUrl;
             QString postAuthorHometown;
             QString postAuthorBio;
+
             QString postGenerator;
             QString postCreatedAt;
+            QString postUpdatedAt;
             QString postTo;
             QString postCC;
             QString postTitle;
             QString postImage;
             QString postText;
-            QString postIsNsfw;
             QString postLikesCount;
             QString postCommentsCount;
             QString postResharesCount;
@@ -360,7 +348,7 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
             bool postIsNew = false;
 
-            postID = objectMap["id"].toString();
+            postID = objectMap.value("id").toString();
             if (newestPostID.isEmpty()) // only first time, for newest post
             {
                 if (this->timelineOffset == 0)
@@ -394,11 +382,9 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
                 }
             }
 
-            postURL = objectMap["url"].toString();
-
-            postType = objectMap["objectType"].toString();
-
-            postIsLiked = objectMap["liked"].toString();
+            postURL = objectMap.value("url").toString();
+            postType = objectMap.value("objectType").toString();
+            postIsLiked = objectMap.value("liked").toString();
 
             QVariantMap authorMap;
             if (itemVerb == "share")
@@ -406,10 +392,10 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
                 postIsShared = "true";
 
                 // Get original author data
-                authorMap = objectMap["author"].toMap();
+                authorMap = objectMap.value("author").toMap();
 
-                postSharedBy = singlePostMap["actor"].toMap()["displayName"].toString();
-                postSharedById = singlePostMap["actor"].toMap()["id"].toString();
+                postSharedBy = singlePostMap.value("actor").toMap().value("displayName").toString();
+                postSharedById = singlePostMap.value("actor").toMap().value("id").toString();
                 if (postSharedById.startsWith("acct:"))
                 {
                     postSharedById.remove(0,5);
@@ -420,59 +406,74 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
                 postIsShared = "false";
 
                 // Not a share, so no original author, just get the actor
-                authorMap = singlePostMap["actor"].toMap();
+                authorMap = singlePostMap.value("actor").toMap();
             }
 
             //// VERY TMP!! Favorites-tmp-stuff
             if (this->timelineType==TimelineTypeFavorites)
             {
-                authorMap = objectMap["author"].toMap();
+                authorMap = objectMap.value("author").toMap();
             }
 
-            postAuthor = authorMap["displayName"].toString();
-            postAuthorAvatar = authorMap["image"].toMap().value("url").toString();
-            postAuthorId = authorMap["id"].toString();
+            postAuthor = authorMap.value("displayName").toString();
+            postAuthorAvatar = authorMap.value("image").toMap().value("url").toString();
+            postAuthorId = authorMap.value("id").toString();
             // For Author ID, remove the first 5 characters from the field, if they are "acct:"
             // (users have acct: in front of the ID, but services like OFG don't)
             if (postAuthorId.startsWith("acct:"))
             {
                 postAuthorId.remove(0,5);
             }
+            postAuthorUrl = authorMap.value("url").toString();
 
-            postAuthorBio = authorMap["summary"].toString();
-            postAuthorHometown = authorMap["location"].toMap().value("displayName").toString();
+            postAuthorHometown = authorMap.value("location").toMap().value("displayName").toString();
+            postAuthorBio = authorMap.value("summary").toString();
+
+            postCreatedAt = objectMap.value("published").toString();
+            postUpdatedAt = objectMap.value("updated").toString();
+            postGenerator = singlePostMap.value("generator").toMap().value("displayName").toString();
 
 
-            postCreatedAt = objectMap["published"].toString();
-            postGenerator = singlePostMap["generator"].toMap().value("displayName").toString();
-
-
-            // Audience
+            // Audience: To
+            QVariantMap postToMap;
             foreach (QVariant postToVariant, singlePostMap.value("to").toList())
             {
-                if (postToVariant.toMap().value("id").toString()
-                  == "http://activityschema.org/collection/public")
+                postToMap = postToVariant.toMap();
+                if (postToMap.value("id").toString()
+                    == "http://activityschema.org/collection/public")
                 {
-                    postTo += tr("Public") + ", ";
+                    postTo += "<b>" + tr("Public") + "</b>, ";
                 }
                 else
                 {
-                    postTo += postToVariant.toMap().value("displayName").toString() + ", ";
+                    QString displayName = postToMap.value("displayName").toString().trimmed();
+                    if (!displayName.isEmpty())
+                    {
+                        postTo += "<a href=\"" + postToMap.value("url").toString() + "\">"
+                                + displayName + "</a>, ";
+                    }
                 }
             }
             postTo.remove(-2, 2); // remove last comma and space
 
-            // CC
+            // and CC
+            QVariantMap postCCMap;
             foreach (QVariant postCCVariant, singlePostMap.value("cc").toList())
             {
-                if (postCCVariant.toMap().value("id").toString()
-                  == "http://activityschema.org/collection/public")
+                postCCMap = postCCVariant.toMap();
+                if (postCCMap.value("id").toString()
+                    == "http://activityschema.org/collection/public")
                 {
-                    postCC += tr("Public") + ", ";
+                    postCC += "<b>" + tr("Public") + "</b>, ";
                 }
                 else
                 {
-                    postCC += postCCVariant.toMap().value("displayName").toString() + ", ";
+                    QString displayName = postCCMap.value("displayName").toString().trimmed();
+                    if (!displayName.isEmpty())
+                    {
+                        postCC += "<a href=\"" + postCCMap.value("url").toString() + "\">"
+                                + displayName + "</a>, ";
+                    }
                 }
             }
             postCC.remove(-2, 2);
@@ -480,67 +481,69 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
 
 
+            ///
             ///
             /// End of "meta"; Start of content
             ///
+            ///
+
+
 
             if (postType == "image")
             {
-                postTitle = objectMap["displayName"].toString();
-
-
                 // See if there's a proxy URL for the image first (for private images)
-                postImage = objectMap["fullImage"].toMap().value("pump_io").toMap().value("proxyURL").toString();
+                postImage = objectMap.value("fullImage").toMap().value("pump_io").toMap().value("proxyURL").toString();
                 qDebug() << "Trying Proxyed fullImage";
 
                 // if that's empty, try the "small" version
                 if (postImage.isEmpty())
                 {
                     qDebug() << "Trying Proxyed thumbnail image";
-                    postImage = objectMap["image"].toMap().value("pump_io").toMap().value("proxyURL").toString();
+                    postImage = objectMap.value("image").toMap().value("pump_io").toMap().value("proxyURL").toString();
                 }
 
                 // If that's also empty, use regular fullImage->url field
                 if (postImage.isEmpty())
                 {
                     qDebug() << "Trying direct fullImage";
-                    postImage = objectMap["fullImage"].toMap().value("url").toString();
+                    postImage = objectMap.value("fullImage").toMap().value("url").toString();
                 }
 
                 // And if that is ALSO empty, use regular image->url
                 if (postImage.isEmpty())
                 {
                     qDebug() << "Trying direct thumbnail image";
-                    postImage = objectMap["image"].toMap().value("url").toString();
+                    postImage = objectMap.value("image").toMap().value("url").toString();
                 }
                 qDebug() << "postImage:" << postImage;
             }
 
+            // Title can be in non-image posts, too!
+            postTitle = objectMap.value("displayName").toString();
+            postText = objectMap.value("content").toString();
 
-            postText = objectMap["content"].toString();
-
-            postLikesCount = objectMap["likes"].toMap().value("totalItems").toString();
-            postCommentsCount = objectMap["replies"].toMap().value("totalItems").toString();
-            postResharesCount = objectMap["shares"].toMap().value("totalItems").toString();
+            postLikesCount = objectMap.value("likes").toMap().value("totalItems").toString();
+            postCommentsCount = objectMap.value("replies").toMap().value("totalItems").toString();
+            postResharesCount = objectMap.value("shares").toMap().value("totalItems").toString();
 
             // get last likes, comments and shares list here, pass to the Post()
-            QVariantList lastLikesList = objectMap["likes"].toMap().value("items").toList();
-            QVariantList lastCommentsList = objectMap["replies"].toMap().value("items").toList();
-            QVariantList lastSharesList = objectMap["shares"].toMap().value("items").toList();
+            QVariantList lastLikesList = objectMap.value("likes").toMap().value("items").toList();
+            QVariantList lastCommentsList = objectMap.value("replies").toMap().value("items").toList();
+            QVariantList lastSharesList = objectMap.value("shares").toMap().value("items").toList();
 
             // Get URL for likes; first, proxyURL if it exists
-            QString postLikesURL = objectMap["likes"].toMap().value("pump_io").toMap().value("proxyURL").toString();
+            QString postLikesURL = objectMap.value("likes").toMap().value("pump_io").toMap().value("proxyURL").toString();
             // If still empty, get regular URL (that means the post is in the same server we are)
             if (postLikesURL.isEmpty())
             {
-                postLikesURL = objectMap["likes"].toMap().value("url").toString();
+                postLikesURL = objectMap.value("likes").toMap().value("url").toString();
             }
 
             // Get URL for comments; first, proxyURL if it exists
-            QString postCommentsURL = objectMap["replies"].toMap().value("pump_io").toMap().value("proxyURL").toString();
+            QString postCommentsURL = objectMap.value("replies").toMap().value("pump_io").toMap().value("proxyURL").toString();
             if (postCommentsURL.isEmpty()) // If still empty, get regular URL
             {
-                postCommentsURL = objectMap["replies"].toMap().value("url").toString();
+                postCommentsURL = objectMap.value("replies").toMap().value("url").toString();
             }
 
 
@@ -552,10 +555,13 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             postData.insert("author",             postAuthor);
             postData.insert("authorAvatar",       postAuthorAvatar);
             postData.insert("authorId",           postAuthorId);
-            postData.insert("authorBio",          postAuthorBio);
+            postData.insert("authorUrl",          postAuthorUrl);
             postData.insert("authorHometown",     postAuthorHometown);
+            postData.insert("authorBio",          postAuthorBio);
+
             postData.insert("generator",          postGenerator);
             postData.insert("createdAt",          postCreatedAt);
+            postData.insert("updatedAt",          postUpdatedAt);
 
             postData.insert("to",                 postTo);
             postData.insert("cc",                 postCC);
@@ -563,7 +569,6 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             postData.insert("title",              postTitle);
             postData.insert("image",              postImage);
             postData.insert("text",               postText);
-            postData.insert("nsfw",               postIsNsfw);
             postData.insert("likesCount",         postLikesCount);
             postData.insert("likesURL",           postLikesURL);
             postData.insert("commentsCount",      postCommentsCount);
@@ -587,24 +592,20 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
                                          lastSharesList,
                                          this);
                 this->postsInTimeline.append(newPost);
-
-                this->splitter->addWidget(newPost);
-
-                this->splitter->setMinimumHeight(this->splitter->height()
-                                                 + newPost->sizeHint().height());
+                this->postsLayout->addWidget(newPost);
 
                 connect(newPost, SIGNAL(postRead()),
                         this, SLOT(decreaseUnreadPostsCount()));
+                connect(newPost, SIGNAL(postEditRequested(QString,QString,QString)),
+                        this, SIGNAL(postEditRequested(QString,QString,QString)));
+                connect(newPost, SIGNAL(commentingOnPost(QWidget*)),
+                        this, SIGNAL(commentingOnPost(QWidget*)));
             }
             else
             {
                 // if there's a "deleted" key, ignore this post
                 qDebug() << "This post was deleted on" << postIsDeleted << " / Not adding.";
             }
-
-
-            // set stretch factor of 1 to each added post
-            this->splitter->setStretchFactor(splitter->count()-1,  1);
         }
         else  // singlePost.type() is not a QVariant::Map
         {
@@ -613,14 +614,7 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
         }
     } // end foreach
 
-    // Add the "extra" empty widget
-    /*
-    this->splitter->addWidget(new QWidget());
-    this->splitter->setMinimumHeight(this->splitter->minimumHeight()
-                                     - splitter->widget(splitter->count()-1)->height());
-    */
 
-    emit scrollToTop(); // ask mainWindow to scroll the QScrollArea containing the timeline to the top
 
     this->previousNewestPostID = newestPostID;
     this->unreadPostsCount = newPostCount;
@@ -628,6 +622,9 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
     this->updateCurrentPageNumber();
 
+    this->resizePosts();
+
+    emit scrollToTop(); // ask mainWindow to scroll the QScrollArea containing the timeline to the top
     emit timelineRendered(this->timelineType, unreadPostsCount);
 
     qDebug() << "setTimeLineContents() /END";
