@@ -26,7 +26,6 @@
 #include <QVBoxLayout>
 #include <QVariantList>
 #include <QMap>
-#include <QSplitter>
 #include <QPushButton>
 
 #include <QDebug>
@@ -46,11 +45,11 @@ public:
     ~TimeLine();
 
     void clearTimeLineContents();
-
     void requestTimelinePage();
-
     void updateCurrentPageNumber();
 
+    void resizePosts();
+
 
     enum TimelineTypes
     {
@@ -71,6 +70,13 @@ signals:
     void timelineRendered(int timelineType, int newPostCount);
     void unreadPostsCountChanged(int timelineType, int newPostCount);
 
+    void postEditRequested(QString originalPostID,
+                           QString contents,
+                           QString title);
+
+    void commentingOnPost(QWidget *commenterWidget);
+
+
 public slots:
     void setTimeLineContents(QVariantList postList, int postsPerPage,
                              QString previousLink, QString nextLink);
@@ -84,9 +90,13 @@ public slots:
 
     void decreaseUnreadPostsCount();
 
+protected:
+    //virtual void resizeEvent(QResizeEvent *event);
+
+
 private:
     QVBoxLayout *mainLayout;
-    QSplitter *splitter;
+    QVBoxLayout *postsLayout;
     QHBoxLayout *bottomLayout;
 
     PumpController *pController;
